@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: audio_directx2_out.c,v 1.4 2005/09/05 17:02:57 valtri Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  *
  * xine audio output plugin using DirectX
@@ -151,7 +149,8 @@ static int buffer_ready(dx2_driver_t *this);
 
 
 /* popup a dialog with error */
-static void error_message(const char *fmt, ...) {
+static void XINE_FORMAT_PRINTF(1, 2)
+	    error_message(const char *fmt, ...) {
   char message[256];
   va_list ap;
 
@@ -957,6 +956,9 @@ static ao_driver_t *open_plugin(audio_driver_class_t *class_gen, const void *dat
   lprintf("open plugin called\n");
 
   this = (dx2_driver_t *)xine_xmalloc(sizeof(dx2_driver_t));
+  if (!this)
+    return NULL;
+
   this->class = class;
 
   this->ao_driver.get_capabilities    = ao_dx2_get_capabilities;
@@ -1008,6 +1010,8 @@ static void *init_class(xine_t *xine, void *data) {
   lprintf("init class\n");
 
   this = (dx2_class_t *)xine_xmalloc(sizeof(dx2_class_t));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = open_plugin;
   this->driver_class.get_identifier  = get_identifier;
@@ -1020,11 +1024,11 @@ static void *init_class(xine_t *xine, void *data) {
 }
 
 
-static ao_info_t ao_info_directx2 = {
+static const ao_info_t ao_info_directx2 = {
   10
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   { PLUGIN_AUDIO_OUT, AO_OUT_DIRECTX2_IFACE_VERSION, "directx2", XINE_VERSION_CODE, &ao_info_directx2, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
