@@ -95,7 +95,7 @@ static int asf_reader_get_8(asf_reader_t *reader, uint8_t *value) {
 static int asf_reader_get_16(asf_reader_t *reader, uint16_t *value) {
   if ((reader->size - reader->pos) < 2)
     return 0;
-  *value = LE_16(reader->buffer + reader->pos);
+  *value = _X_LE_16(reader->buffer + reader->pos);
   reader->pos += 2;
   return 1;
 }
@@ -103,7 +103,7 @@ static int asf_reader_get_16(asf_reader_t *reader, uint16_t *value) {
 static int asf_reader_get_32(asf_reader_t *reader, uint32_t *value) {
   if ((reader->size - reader->pos) < 4)
     return 0;
-  *value = LE_32(reader->buffer + reader->pos);
+  *value = _X_LE_32(reader->buffer + reader->pos);
   reader->pos += 4;
   return 1;
 }
@@ -111,7 +111,7 @@ static int asf_reader_get_32(asf_reader_t *reader, uint32_t *value) {
 static int asf_reader_get_64(asf_reader_t *reader, uint64_t *value) {
   if ((reader->size - reader->pos) < 8)
     return 0;
-  *value = LE_64(reader->buffer + reader->pos);
+  *value = _X_LE_64(reader->buffer + reader->pos);
   reader->pos += 8;
   return 1;
 }
@@ -142,9 +142,9 @@ static uint8_t *asf_reader_get_bytes(asf_reader_t *reader, size_t size) {
 static char *asf_reader_get_string(asf_reader_t *reader, size_t size, iconv_t cd) {
   char *inbuf, *outbuf;
   size_t inbytesleft, outbytesleft;
-  char scratch[2048]; 
+  char scratch[2048];
 
-  if ((reader->size - reader->pos) < size)
+  if ((size == 0) ||((reader->size - reader->pos) < size))
     return NULL;
 
   inbuf = (char *)reader->buffer + reader->pos;
@@ -187,9 +187,9 @@ static size_t asf_reader_get_size(asf_reader_t *reader) {
 void asf_get_guid(uint8_t *buffer, GUID *value) {
   int i;
 
-  value->Data1 = LE_32(buffer);
-  value->Data2 = LE_16(buffer + 4);
-  value->Data3 = LE_16(buffer + 6);
+  value->Data1 = _X_LE_32(buffer);
+  value->Data2 = _X_LE_16(buffer + 4);
+  value->Data3 = _X_LE_16(buffer + 6);
   for(i = 0; i < 8; i++) {
     value->Data4[i] = *(buffer + i + 8);
   }
@@ -399,7 +399,7 @@ static int asf_header_parse_stream_extended_properties(asf_header_t *header, uin
   if (asf_stream_extension->stream_name_count) {
     asf_stream_extension->stream_names = malloc (asf_stream_extension->stream_name_count * sizeof(void*));
     for (i = 0; i < asf_stream_extension->stream_name_count; i++) {
-      uint16_t lang_index, length;
+      uint16_t lang_index, length = 0;
       asf_reader_get_16(&reader, &lang_index);
       asf_reader_get_16(&reader, &length);
       asf_stream_extension->stream_names[i] = (char*)asf_reader_get_bytes(&reader, length); /* store them */
@@ -411,7 +411,7 @@ static int asf_header_parse_stream_extended_properties(asf_header_t *header, uin
     for (i = 0; i < asf_stream_extension->payload_extension_system_count; i++) {
       GUID guid;
       uint16_t data_size;
-      uint32_t length;
+      uint32_t length = 0;
       asf_reader_get_guid(&reader, &guid);
       asf_reader_get_16(&reader, &data_size);
       asf_reader_get_32(&reader, &length);
@@ -427,7 +427,7 @@ static int asf_header_parse_stream_extended_properties(asf_header_t *header, uin
   /* embeded stream properties */
   if (asf_reader_get_size(&reader) >= 24) {
     GUID guid;
-    uint64_t object_length;
+    uint64_t object_length = 0;
 
     asf_reader_get_guid(&reader, &guid);
     asf_reader_get_64(&reader, &object_length);
@@ -490,8 +490,8 @@ static int asf_header_parse_stream_bitrate_properties(asf_header_t *header_pub,
   lprintf ("  bitrate count: %d\n", bitrate_count);
 
   for(i = 0; i < bitrate_count; i++) {
-    uint16_t flags;
-    uint32_t bitrate;
+    uint16_t flags = 0;
+    uint32_t bitrate = 0;
     int stream_number;
     uint8_t *bitrate_pointer;
 
@@ -533,7 +533,7 @@ static int asf_header_parse_header_extension(asf_header_t *header, uint8_t *buff
 
     GUID guid;
     int object_id;
-    uint64_t object_length, object_data_length;
+    uint64_t object_length = 0, object_data_length;
 
     if (asf_reader_get_size(&reader) < 24) {
       printf("invalid buffer size\n");
@@ -599,6 +599,12 @@ static int asf_header_parse_content_description(asf_header_t *header_pub, uint8_
   content->description = asf_reader_get_string(&reader, description_length, header->iconv_cd);
   content->rating = asf_reader_get_string(&reader, rating_length, header->iconv_cd);
 
+  lprintf("title: %d chars: \"%s\"\n", title_length, content->title);
+  lprintf("author: %d chars: \"%s\"\n", author_length, content->author);
+  lprintf("copyright: %d chars: \"%s\"\n", copyright_length, content->copyright);
+  lprintf("description: %d chars: \"%s\"\n", description_length, content->description);
+  lprintf("rating: %d chars: \"%s\"\n", rating_length, content->rating);
+
   header->pub.content = content;
   return 1;
 }
@@ -638,7 +644,7 @@ asf_header_t *asf_header_new (uint8_t *buffer, int buffer_len) {
 
     GUID guid;
     int object_id;
-    uint64_t object_length, object_data_length;
+    uint64_t object_length = 0, object_data_length;
 
     if (asf_reader_get_size(&reader) < 24) {
       printf("invalid buffer size\n");
