@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
@@ -64,8 +64,6 @@
  *   - scan through all 768 palette bytes
  *     - if any bytes exceed 63, do not shift the bytes at all before
  *       transmitting them to the video decoder
- *
- * $Id: demux_idcin.c,v 1.54 2007/01/19 00:26:40 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -191,7 +189,7 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
     this->status = DEMUX_FINISHED;
     return this->status;
   }
-  remaining_sample_bytes = LE_32(&preamble[0]) - 4;
+  remaining_sample_bytes = _X_LE_32(&preamble[0]) - 4;
 
   lprintf("dispatching %d video bytes\n", remaining_sample_bytes);
   while (remaining_sample_bytes) {
@@ -222,7 +220,7 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
     if (!remaining_sample_bytes)
       buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-    lprintf("sending video buf with %d bytes, %lld pts\n", buf->size, buf->pts);
+    lprintf("sending video buf with %d bytes, %"PRId64" pts\n", buf->size, buf->pts);
     this->video_fifo->put(this->video_fifo, buf);
   }
 
@@ -263,7 +261,7 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
       if (!remaining_sample_bytes)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
-      lprintf("sending audio buf with %d bytes, %lld pts\n", buf->size, buf->pts);
+      lprintf("sending audio buf with %d bytes, %"PRId64" pts\n", buf->size, buf->pts);
       this->audio_fifo->put(this->audio_fifo, buf);
     }
   }
@@ -293,28 +291,28 @@ static int open_idcin_file(demux_idcin_t *this) {
    */
 
   /* check the width */
-  bih->biWidth = LE_32(&header[0]);
+  bih->biWidth = _X_LE_32(&header[0]);
   if ((bih->biWidth == 0) || (bih->biWidth > 1024))
     return 0;
 
   /* check the height */
-  bih->biHeight = LE_32(&header[4]);
+  bih->biHeight = _X_LE_32(&header[4]);
   if ((bih->biHeight == 0) || (bih->biHeight > 1024))
     return 0;
 
   /* check the audio sample rate */
-  this->wave.nSamplesPerSec = LE_32(&header[8]);
+  this->wave.nSamplesPerSec = _X_LE_32(&header[8]);
   if ((this->wave.nSamplesPerSec != 0) &&
      ((this->wave.nSamplesPerSec < 8000) || (this->wave.nSamplesPerSec > 48000)))
     return 0;
 
   /* check the audio bytes/sample */
-  this->wave.wBitsPerSample = LE_32(&header[12]) * 8;
+  this->wave.wBitsPerSample = _X_LE_32(&header[12]) * 8;
   if (this->wave.wBitsPerSample > 16)
     return 0;
 
   /* check the audio channels */
-  this->wave.nChannels = LE_32(&header[16]);
+  this->wave.nChannels = _X_LE_32(&header[16]);
   if (this->wave.nChannels > 2)
     return 0;
 
