@@ -15,12 +15,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
- * $Id: demux_mpeg.c,v 1.151 2007/01/19 00:26:40 dgp85 Exp $
- *
  * demultiplexer for mpeg 1/2 program streams
  * reads streams of variable blocksizes
  */
@@ -140,8 +138,8 @@ static void find_mdat_atom(input_plugin_t *input, off_t *mdat_offset,
       ATOM_PREAMBLE_SIZE)
       break;
 
-    atom_size = BE_32(&atom_preamble[0]);
-    atom = BE_32(&atom_preamble[4]);
+    atom_size = _X_BE_32(&atom_preamble[0]);
+    atom = _X_BE_32(&atom_preamble[4]);
 
     if (atom == MDAT_ATOM) {
       *mdat_offset = input->get_current_pos(input) - ATOM_PREAMBLE_SIZE;
@@ -165,9 +163,9 @@ static void find_mdat_atom(input_plugin_t *input, off_t *mdat_offset,
         ATOM_PREAMBLE_SIZE)
         break;
 
-      atom_size = BE_32(&atom_preamble[0]);
+      atom_size = _X_BE_32(&atom_preamble[0]);
       atom_size <<= 32;
-      atom_size |= BE_32(&atom_preamble[4]);
+      atom_size |= _X_BE_32(&atom_preamble[4]);
       atom_size -= ATOM_PREAMBLE_SIZE * 2;
     } else
       atom_size -= ATOM_PREAMBLE_SIZE;
@@ -310,7 +308,7 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
         check_newpts( this, this->pts, PTS_VIDEO );
       */
       this->video_fifo->put (this->video_fifo, buf);
-      lprintf ("SPU SVCD PACK (pts: %lld, spu id: %d) put on FIFO\n", 
+      lprintf ("SPU SVCD PACK (pts: %"PRId64", spu id: %d) put on FIFO\n", 
 	       buf->pts, spu_id);
 
       return;
@@ -1157,7 +1155,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     }
 
     /* special case for MPEG streams with a RIFF header */
-    fourcc_tag = BE_32(&buf[0]);
+    fourcc_tag = _X_BE_32(&buf[0]);
     if (fourcc_tag == RIFF_TAG) {
       uint8_t large_buf[1024];
       
@@ -1165,7 +1163,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
         free(this);
         return NULL;
       }
-      fourcc_tag = BE_32(&large_buf[8]);
+      fourcc_tag = _X_BE_32(&large_buf[8]);
       /* disregard the RIFF file if it is certainly a better known
        * format like AVI or WAVE */
       if ((fourcc_tag == WAVE_TAG) ||
@@ -1185,7 +1183,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 	if (input->read(input, large_buf, 1024) != 1024)
 	  break;
 	for (j = 0; j < 1024 - 4; j++) {
-	  if (BE_32(&large_buf[j]) == MPEG_MARKER) {
+	  if (_X_BE_32(&large_buf[j]) == MPEG_MARKER) {
 	    ok = 1;
 	    break;
 	  }
