@@ -15,13 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_mpeg_block.c,v 1.219 2007/01/19 01:05:24 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * demultiplexer for mpeg 1/2 program streams
  * used with fixed blocksize devices (like dvd/vcd)
- *
  */
  
 #ifdef HAVE_CONFIG_H
@@ -458,7 +455,7 @@ static int32_t parse_program_stream_pack_header(demux_mpeg_block_t *this, uint8_
     this->scr += ( (p[8] & 0x03 << 7) | (p[9] & 0xFE >> 1) );
     */
 
-    lprintf ("SCR=%lld\n", this->scr);
+    lprintf ("SCR=%"PRId64"\n", this->scr);
 
     /* mux_rate */
 
@@ -525,7 +522,7 @@ static int32_t parse_private_stream_2(demux_mpeg_block_t *this, uint8_t *p, buf_
     this->last_begin_time = buf->extra_info->input_time;
   }
     
-  lprintf ("NAV packet, start pts = %lld, end_pts = %lld\n",
+  lprintf ("NAV packet, start pts = %"PRId64", end_pts = %"PRId64"\n",
            start_pts, end_pts);
 
   if (this->nav_last_end_pts != start_pts && !this->preview_mode) {
@@ -664,7 +661,7 @@ static int32_t parse_pes_for_pts(demux_mpeg_block_t *this, uint8_t *p, buf_eleme
       this->pts |=  p[12]         <<  7 ;
       this->pts |= (p[13] & 0xFE) >>  1 ;
 
-      lprintf ("pts = %lld\n", this->pts);
+      lprintf ("pts = %"PRId64"\n", this->pts);
 
     } else
       this->pts = 0;
@@ -731,7 +728,7 @@ static int32_t parse_private_stream_1(demux_mpeg_block_t *this, uint8_t *p, buf_
         check_newpts( this, this->pts, PTS_VIDEO );
       */
       this->video_fifo->put (this->video_fifo, buf);    
-      lprintf ("SPU SVCD PACK (%lld, %d) put on fifo\n", this->pts, spu_id);
+      lprintf ("SPU SVCD PACK (%"PRId64", %d) put on fifo\n", this->pts, spu_id);
       
       return -1;
     }
@@ -749,7 +746,7 @@ static int32_t parse_private_stream_1(demux_mpeg_block_t *this, uint8_t *p, buf_
         check_newpts( this, this->pts, PTS_VIDEO );
       */      
       this->video_fifo->put (this->video_fifo, buf);    
-      lprintf ("SPU CVD PACK (%lld, %d) put on fifo\n", this->pts, spu_id);
+      lprintf ("SPU CVD PACK (%"PRId64", %d) put on fifo\n", this->pts, spu_id);
       
       return -1;
     }
@@ -1146,7 +1143,7 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
 	count ++;
 	
 	/*
-	printf ("demux_mpeg_block: stream_id %02x, pos: %lld, pts: %d, cur_rate = %d, overall rate : %d\n", 
+	printf ("demux_mpeg_block: stream_id %02x, pos: %"PRId64", pts: %d, cur_rate = %d, overall rate : %d\n", 
 		stream_id, pos, pts, cur_rate, rate); 
 	*/
       }
