@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_mpeg_pes.c,v 1.35 2006/06/17 15:20:56 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * demultiplexer for mpeg 2 PES (Packetized Elementary Streams)
  * reads streams of variable blocksizes
@@ -25,7 +23,6 @@
  * 1-7-2003 New implementation of mpeg 2 PES demuxers.
  *   (c) 2003 James Courtier-Dutton James@superbug.demon.co.uk
  *   This code might also decode normal MPG files.
- *
  */
  
 #ifdef HAVE_CONFIG_H
@@ -75,25 +72,29 @@ typedef struct demux_mpeg_pes_s {
 
   char                  cur_mrl[256];
 
-  uint8_t              *scratch, *scratch_base;
+  uint8_t              *scratch;
+  void                 *scratch_base;
 
   int64_t               nav_last_end_pts;
   int64_t               nav_last_start_pts;
   int64_t               last_pts[2];
-  int                   send_newpts;
-  int                   preview_mode;
-  int                   buf_flag_seek;
   int64_t               scr;
   uint32_t              packet_len;
+  uint32_t              stream_id;
+
   int64_t               pts;
   int64_t               dts;
-  uint32_t              stream_id;
-  int32_t               mpeg1;
-  int32_t               wait_for_program_stream_pack_header;
 
+  uint8_t               send_newpts:1;
+  uint8_t               buf_flag_seek:1;
+  uint8_t               preview_mode:1;
+  uint8_t               mpeg1:1;
+  uint8_t               wait_for_program_stream_pack_header:1;
+  uint8_t               mpeg12_h264_detected:2;
+  
+  int                   last_begin_time;
   int64_t               last_cell_time;
   off_t                 last_cell_pos;
-  int                   last_begin_time;
 
   uint8_t               preview_data[ MAX_PREVIEW_SIZE ];
   off_t                 preview_size, preview_done;
@@ -132,13 +133,79 @@ static int32_t parse_IEC14496_FlexMux_stream(demux_mpeg_pes_t *this, uint8_t *p,
 static int32_t parse_program_stream_directory(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf);
 static int32_t parse_program_stream_pack_header(demux_mpeg_pes_t *this, uint8_t *p, buf_element_t *buf);
 
-static void check_newpts( demux_mpeg_pes_t *this, int64_t pts, int video )
+static int detect_pts_discontinuity( demux_mpeg_pes_t *this, int64_t pts, int video )
 {
   int64_t diff;
-  
+
+  /* discontinuity detection is difficult to implement in the demuxer as it gets
+   * for example video packets in decoding order and there can be multiple audio
+   * and video tracks. So for simplicity, let's just deal with a single audio and
+   * a single video track.
+   *
+   * To start with, let's have a look at the audio and video track independently.
+   * Whenever pts differs from last_pts[video] by at least WRAP_THRESHOLD, a jump
+   * in pts is detected. Such a jump can happen for example when the pts counter
+   * overflows, as shown below (video decoding order ignored for simplicity; the
+   * variable values are shown after returning from the below function check_newpts;
+   * an asterisk means that this value has been cleared (see check_newpts)):
+   *
+   *         pts: 7v 7a 8v 9v 9a : 0v 1v 1a 2v 3v 3a 4v
+   * last_pts[0]: 6  7  7  7  9  : *  *  1  1  1  3  3
+   * last_pts[1]: 7  7  8  9  9  : 0  1  1  2  3  3  4
+   *                             | |     |
+   *                             | |     +--- audio pts wrap ignored
+   *                             | +--------- video pts wrap detected
+   *                             +----------- pts wrap boundary
+   */
   diff = pts - this->last_pts[video];
-  
-  if( pts && (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD) ) ) {
+
+  if (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD)
+    return 1;
+
+  /* but the above code can cause a huge delay while replaying when audio and video
+   * track are not aligned on a common pts wrap boundery, as shown below:
+   *
+   *         pts: 7v 8v 7a 9v : 0v 9a 1v 2v : 1a 3v 4v 3a
+   * last_pts[0]: 6  6  7  7  : *  9  9  9  : 1  1  1  3
+   * last_pts[1]: 7  8  8  9  : 0  0  1  2  : *  3  4  4
+   *                          | |  |        | |
+   *                          | |  |        | +--- audio pts wrap detected
+   *                          | |  |        +----- audio pts wrap boundary
+   *                          | |  +-------------- audio packet causes a huge delay
+   *                          | +----------------- video pts wrap detected
+   *                          +------------------- video pts wrap boundery
+   *
+   * So there is the need to compare audio track pts against video track pts
+   * to detect when pts values are in between pts wrap bounderies, where a
+   * jump needs to be detected too, as shown below:
+   *
+   *         pts: 7v 8v 7a 9v : 0v 9a 1v 2v : 1a 3v 4v 3a
+   * last_pts[0]: 6  6  7  7  : *  9  *  *  : 1  1  1  3
+   * last_pts[1]: 7  8  8  9  : 0  *  1  2  : 2  3  4  4
+   *                          | |  |  |     | |
+   *                          | |  |  |     | +--- (audio pts wrap ignored)
+   *                          | |  |  |     +----- audio pts wrap boundary
+   *                          | |  |  +----------- video pts wrap detected
+   *                          | |  +-------------- audio pts wrap detected
+   *                          | +----------------- video pts wrap detected
+   *                          +------------------- (video pts wrap boundery)
+   *
+   * Basically, it's almost the same test like above, but against the other track's
+   * pts value and with a different limit. As the pts counter is a 33 bit unsigned
+   * integer, we choose 2^31 as limit (2^32 would require the tracks to be aligned).
+   */
+  diff = pts - this->last_pts[1-video];
+
+  if (this->last_pts[1-video] && abs(diff)>(1u<<31))
+    return 1;
+
+  /* no discontinuity detected */
+  return 0;
+}
+ 
+static void check_newpts( demux_mpeg_pes_t *this, int64_t pts, int video )
+{
+  if( pts && (this->send_newpts || detect_pts_discontinuity(this, pts, video) ) ) {
 
     /* check if pts is outside nav pts range. any stream without nav must enter here. */
     if( pts > this->nav_last_end_pts || pts < this->nav_last_start_pts )
@@ -155,47 +222,13 @@ static void check_newpts( demux_mpeg_pes_t *this, int64_t pts, int video )
     } else {
       lprintf("no wrap detected\n" );
     }
-    
+
+    /* clear pts on the other track to avoid detecting the same discontinuity again */
     this->last_pts[1-video] = 0;
   }
   
   if( pts )
-  {
-    /* don't detect a discontinuity only for video respectively audio. It's also a discontinuity
-       indication when audio and video pts differ to much e. g. when a pts wrap happens.
-       The original code worked well when the wrap happend like this:
-
-       V7 A7 V8 V9 A9 Dv V0 V1 da A1 V2 V3 A3 V4
-       
-       Legend:
-       Vn = video packet with timestamp n
-       An = audio packet with timestamp n
-       Dv = discontinuity detected on following video packet
-       Da = discontinuity detected on following audio packet
-       dv = discontinuity detected on following video packet but ignored
-       da = discontinuity detected on following audio packet but ignored
-
-       But with a certain delay between audio and video packets (e. g. the way DVB-S broadcasts
-       the packets) the code didn't work:
-
-       V7 V8 A7 V9 Dv V0 _A9_ V1 V2 Da _A1_ V3 V4 A3
-
-       Packet A9 caused audio to jump forward and A1 caused it to jump backward with inserting
-       a delay of almoust 26.5 hours!
-
-       The new code gives the following sequences for the above examples:
-       
-       V7 A7 V8 V9 A9 Dv V0 V1 A1 V2 V3 A3 V4
-
-       V7 V8 A7 V9 Dv V0 Da A9 Dv V1 V2 A1 V3 V4 A3
-
-       After proving this code it should be cleaned up to use just a single variable "last_pts". */
-    
-/*
     this->last_pts[video] = pts;
-*/    
-    this->last_pts[video] = this->last_pts[1-video] = pts;
-  }
 }
 
 static off_t read_data(demux_mpeg_pes_t *this, uint8_t *buf, off_t nlen)
@@ -416,6 +449,9 @@ static int32_t parse_padding_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elem
     done += i;
   }
 
+  /* trigger detection of MPEG 1/2 respectively H.264 content */
+  this->mpeg12_h264_detected = 0;
+
   buf->free_buffer(buf);
   return this->packet_len + 6;
 }
@@ -571,7 +607,7 @@ static int32_t parse_program_stream_pack_header(demux_mpeg_pes_t *this, uint8_t
     this->scr += ( (p[8] & 0x03 << 7) | (p[9] & 0xFE >> 1) );
     */
 
-    lprintf ("SCR=%lld\n", this->scr);
+    lprintf ("SCR=%"PRId64"\n", this->scr);
 
     /* mux_rate */
 
@@ -648,7 +684,7 @@ static int32_t parse_private_stream_2(demux_mpeg_pes_t *this, uint8_t *p, buf_el
     this->last_begin_time = buf->extra_info->input_time;
   }
     
-  lprintf ("NAV packet, start pts = %lld, end_pts = %lld\n",
+  lprintf ("NAV packet, start pts = %"PRId64", end_pts = %"PRId64"\n",
            start_pts, end_pts);
 
   if (this->nav_last_end_pts != start_pts && !this->preview_mode) {
@@ -794,7 +830,7 @@ static int32_t parse_pes_for_pts(demux_mpeg_pes_t *this, uint8_t *p, buf_element
       this->pts |= (int64_t)  p[12]         <<  7 ;
       this->pts |= (int64_t) (p[13] & 0xFE) >>  1 ;
 
-      lprintf ("pts = %lld\n", this->pts);
+      lprintf ("pts = %"PRId64"\n", this->pts);
 
     } else
       this->pts = 0;
@@ -861,7 +897,7 @@ static int32_t parse_private_stream_1(demux_mpeg_pes_t *this, uint8_t *p, buf_el
         check_newpts( this, this->pts, PTS_VIDEO );
       */
       this->video_fifo->put (this->video_fifo, buf);    
-      lprintf ("SPU SVCD PACK (%lld, %d) put on fifo\n", this->pts, spu_id);
+      lprintf ("SPU SVCD PACK (%"PRId64", %d) put on fifo\n", this->pts, spu_id);
       
       return this->packet_len + result;
     }
@@ -879,7 +915,7 @@ static int32_t parse_private_stream_1(demux_mpeg_pes_t *this, uint8_t *p, buf_el
         check_newpts( this, this->pts, PTS_VIDEO );
       */      
       this->video_fifo->put (this->video_fifo, buf);    
-      lprintf ("SPU CVD PACK (%lld, %d) put on fifo\n", this->pts, spu_id);
+      lprintf ("SPU CVD PACK (%"PRId64", %d) put on fifo\n", this->pts, spu_id);
       
       return this->packet_len + result;
     }
@@ -1061,21 +1097,105 @@ static int32_t parse_video_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elemen
   uint32_t todo_length=0;
   uint32_t i;
   uint32_t chunk_length;
+  int buf_type = BUF_VIDEO_MPEG;
+  int payload_size;
 
   result = parse_pes_for_pts(this, p, buf);
   if (result < 0) return -1;
 
   p += result;
 
-  buf->content   = p;
+  buf->content = p;
+  payload_size = buf->max_size - result;
+  if (payload_size > this->packet_len)
+    payload_size = this->packet_len;
 
-  if (this->packet_len <= (buf->max_size - 6)) {
-    buf->size      = this->packet_len;
-  } else {
-    buf->size      = buf->max_size - result;
-    todo_length    = this->packet_len - buf->size;
+  /* H.264 broadcasts via DVB-S use standard video PES packets,
+     so there is no way other than scanning the video data to
+     detect whether BUF_VIDEO_H264 needs to be used.
+     For performance reasons, this is not a general scanner for
+     H.264 content, as this kind of data format is likely to be
+     used only by VDR and VDR will ensure that an AUD-NAL unit
+     will be at the beginning of the PES packet's payload.
+     To minimize false hits, the whole payload is scanned for
+     MPEG 1/2 start codes, so there is only a little chance left
+     that a MPEG 1/2 slice 9 start code will be considered as a
+     H.264 access unit delimiter (should only happen after a seek).
+
+     Meaning of bit 0 and 1 of mpeg12_h264_detected:
+     Bit 0: H.264 access unit delimiter seen
+     Bit 1: H.264 AUD seen again or MPEG 1/2 start code seen
+
+     For performance reasons, the scanner is only active until
+     a H.264 AUD has been seen a second time or a MPEG 1/2 start
+     code has been seen. The scanner get's activated initially
+     (e. g. when opening the stream), after seeking or when VDR
+     sends a padding packet.
+     Until the scanner is convinced of it's decision by setting
+     bit 1, the default behaviour is to assume MPEG 1/2 unless
+     an AUD has been found at the beginning of the payload.
+   */
+  if (this->mpeg12_h264_detected < 2) {
+    uint8_t *pp = p + 2, *pp_limit = p + payload_size - 1;
+    while (0 < pp && pp < pp_limit) {
+      if (pp[0] == 0x01 && pp[-1] == 0x00 && pp[-2] == 0x00) {
+        if (pp[1] >= 0x80 || !pp[1]) { /* MPEG 1/2 start code */
+          this->mpeg12_h264_detected = 2;
+          break;
+        } else {
+          int nal_type_code = pp[1] & 0x1f;
+          if (nal_type_code == 9 && pp == (p + 2)) { /* access unit delimiter */
+            if (this->mpeg12_h264_detected == 1) {
+              this->mpeg12_h264_detected = 3;
+              break;
+            }
+            this->mpeg12_h264_detected = 1;
+          }
+        }
+      }
+      pp++;
+      pp = memchr(pp, 0x01, pp_limit - pp);
+    }
+    lprintf("%s%c\n", (this->mpeg12_h264_detected & 1) ? "H.264" : "MPEG1/2", (this->mpeg12_h264_detected & 2) ? '!' : '?');
   }
-  buf->type      = BUF_VIDEO_MPEG;
+
+  /* when an H.264 AUD is seen, we first need to tell the decoder that the
+     previous frame was complete.
+   */
+  if (this->mpeg12_h264_detected & 1) {
+    buf_type = BUF_VIDEO_H264;
+    /* omit sending BUF_FLAG_FRAME_END for the first AUD occurence */
+    if (this->mpeg12_h264_detected > 2) {
+      int nal_type_code = -1;
+      if (payload_size >= 4 && p[2] == 0x01 && p[1] == 0x00 && p[0] == 0x00)
+        nal_type_code = p[3] & 0x1f; 
+      if (nal_type_code == 9) { /* access unit delimiter */
+        buf_element_t *b = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+        b->content       = b->mem;
+        b->size          = 0;
+        b->pts           = 0;
+        b->type          = buf_type;
+        b->decoder_flags = BUF_FLAG_FRAME_END;
+        this->video_fifo->put (this->video_fifo, b);
+      }
+    }
+  }
+
+  if (this->packet_len <= (buf->max_size - result)) {
+    buf->size = this->packet_len;
+    /* VDR ensures that H.264 still images end with an end of sequence NAL unit. We
+       need to detect this to inform the decoder that the current frame is complete.
+     */
+    if (this->mpeg12_h264_detected & 1) {
+      uint8_t *t = buf->content + buf->size;
+      if (buf->size >=4 && t[-1] == 10 && t[-2] == 0x01 && t[-3] == 0x00 && t[-4] == 0x00) /* end of sequence */
+        buf->decoder_flags = BUF_FLAG_FRAME_END;
+    }
+  } else {
+    buf->size    = buf->max_size - result;
+    todo_length  = this->packet_len - buf->size;
+  }
+  buf->type      = buf_type;
   buf->pts       = this->pts;
   buf->decoder_info[0] = this->pts - this->dts;
   if( !this->preview_mode )
@@ -1097,10 +1217,20 @@ static int32_t parse_video_stream(demux_mpeg_pes_t *this, uint8_t *p, buf_elemen
       }
     buf->content   = buf->mem;
     buf->size      = chunk_length;
-    buf->type      = BUF_VIDEO_MPEG;
+    buf->type      = buf_type;
     buf->pts       = 0;
-    this->video_fifo->put (this->video_fifo, buf);
     todo_length -= chunk_length;
+
+    /* VDR ensures that H.264 still images end with an end of sequence NAL unit. We
+       need to detect this to inform the decoder that the current frame is complete.
+     */
+    if ((this->mpeg12_h264_detected & 1) && todo_length <= 0) {
+      uint8_t *t = buf->content + buf->size;
+      if (buf->size >= 4 && t[-1] == 10 && t[-2] == 0x01 && t[-3] == 0x00 && t[-4] == 0x00) /* end of sequence */
+        buf->decoder_flags = BUF_FLAG_FRAME_END;
+    }
+
+    this->video_fifo->put (this->video_fifo, buf);
   }
 
   lprintf ("MPEG Video PACK put on fifo\n");
@@ -1282,7 +1412,7 @@ static int demux_mpeg_pes_estimate_rate (demux_mpeg_pes_t *this) {
 	count ++;
 	
 	/*
-	printf ("demux_mpeg_pes: stream_id %02x, pos: %lld, pts: %d, cur_rate = %d, overall rate : %d\n", 
+	printf ("demux_mpeg_pes: stream_id %02x, pos: %"PRId64", pts: %d, cur_rate = %d, overall rate : %d\n", 
 		stream_id, pos, pts, cur_rate, rate); 
 	*/
       }
@@ -1425,6 +1555,8 @@ static int demux_mpeg_pes_seek (demux_plugin_t *this_gen,
   } else {
     this->buf_flag_seek = 1;
     this->nav_last_end_pts = this->nav_last_start_pts = 0;
+    /* trigger detection of MPEG 1/2 respectively H.264 content */
+    this->mpeg12_h264_detected = 0;
     _x_demux_flush_engine(this->stream);
   }
   
@@ -1499,7 +1631,9 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->status     = DEMUX_FINISHED;
   /* Don't start demuxing stream until we see a program_stream_pack_header */
   /* We need to system header in order to identify is the stream is mpeg1 or mpeg2. */
-  this->wait_for_program_stream_pack_header=1;
+  this->wait_for_program_stream_pack_header = 1;
+  /* trigger detection of MPEG 1/2 respectively H.264 content */
+  this->mpeg12_h264_detected = 0;
 
   this->preview_size = 0;
 
@@ -1593,11 +1727,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   break;
 
   case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
+    const char *const mrl = input->get_mrl (input);
+    const char *const ending = strrchr(mrl, '.');
 
     if (!ending) {
       free (this->scratch_base);
@@ -1629,19 +1760,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "mpeg pes demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "MPEG_PES";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "pes";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
 
@@ -1676,7 +1807,7 @@ static const demuxer_info_t demux_info_mpeg_pes = {
   10                       /* priority */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_DEMUX, 26, "mpeg_pes", XINE_VERSION_CODE, &demux_info_mpeg_pes, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
