@@ -15,14 +15,11 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
- * $Id: demux_ogg.c,v 1.174 2007/01/23 23:20:23 hadess Exp $
- *
  * demultiplexer for ogg streams
- *
  */
 /* 2003.02.09 (dilb) update of the handling for audio/video infos for strongarm cpus. */
 
@@ -126,17 +123,20 @@ typedef struct demux_ogg_s {
   input_plugin_t       *input;
   int                   status;
 
+  int                   frame_duration;
+
 #ifdef HAVE_THEORA
   theora_info           t_info;
   theora_comment        t_comment;
 #endif
 
-  int                   frame_duration;
-
   ogg_sync_state        oy;
   ogg_page              og;
 
   int64_t               start_pts;
+  int64_t               last_pts[2];
+
+  int                   time_length;
 
   int                   num_streams;
   stream_info_t        *si[MAX_STREAMS];   /* stream info */
@@ -148,16 +148,14 @@ typedef struct demux_ogg_s {
 
   off_t                 avg_bitrate;
 
-  int64_t               last_pts[2];
-  int                   send_newpts;
-  int                   buf_flag_seek;
-  int                   keyframe_needed;
-  int                   ignore_keyframes;
-  int                   time_length;
-
   char                 *title;
   chapter_info_t       *chapter_info;
   xine_event_queue_t   *event_queue;
+
+  uint8_t               send_newpts:1;
+  uint8_t               buf_flag_seek:1;
+  uint8_t               keyframe_needed:1;
+  uint8_t               ignore_keyframes:1;
 } demux_ogg_t ;
 
 typedef struct {
@@ -235,13 +233,19 @@ static int64_t get_pts (demux_ogg_t *this, int stream_num , int64_t granulepos )
 static int read_ogg_packet (demux_ogg_t *this) {
   char *buffer;
   long bytes;
+  long total = 0;
   while (ogg_sync_pageout(&this->oy,&this->og)!=1) {
     buffer = ogg_sync_buffer(&this->oy, CHUNKSIZE);
     bytes  = this->input->read(this->input, buffer, CHUNKSIZE);
-    ogg_sync_wrote(&this->oy, bytes);
-    if (bytes < CHUNKSIZE/2) {
-      return 0;
+    if (bytes == 0) {
+      if (total == 0) {
+        lprintf("read_ogg_packet read nothing\n");
+        return 0;
+      }
+      break;
     }
+    ogg_sync_wrote(&this->oy, bytes);
+    total += bytes;
   }
   return 1;
 }
@@ -718,7 +722,7 @@ static void send_ogg_buf (demux_ogg_t *this,
 
     buf->size = 12 + op->bytes + 1;
 
-    lprintf ("CMML stream %d (bytes=%ld): PTS %lld: %s\n",
+    lprintf ("CMML stream %d (bytes=%ld): PTS %"PRId64": %s\n",
              stream_num, op->bytes, buf->pts, str);
 
     this->video_fifo->put (this->video_fifo, buf);
@@ -861,14 +865,14 @@ static void decode_video_header (demux_ogg_t *this, const int stream_num, ogg_pa
   locsubtype = *((uint32_t *)&op->packet[9]);
 
   /* everything else little endian */
-  locsize = LE_32(&op->packet[13]);
-  loctime_unit = LE_64(&op->packet[17]);
-  locsamples_per_unit = LE_64(&op->packet[25]);
-  locdefault_len = LE_32(&op->packet[33]);
-  locbuffersize = LE_32(&op->packet[37]);
-  locbits_per_sample = LE_16(&op->packet[41]);
-  locwidth = LE_32(&op->packet[45]);
-  locheight = LE_32(&op->packet[49]);
+  locsize = _X_LE_32(&op->packet[13]);
+  loctime_unit = _X_LE_64(&op->packet[17]);
+  locsamples_per_unit = _X_LE_64(&op->packet[25]);
+  locdefault_len = _X_LE_32(&op->packet[33]);
+  locbuffersize = _X_LE_32(&op->packet[37]);
+  locbits_per_sample = _X_LE_16(&op->packet[41]);
+  locwidth = _X_LE_32(&op->packet[45]);
+  locheight = _X_LE_32(&op->packet[49]);
 
   lprintf ("direct show filter created stream detected, hexdump:\n");
 #ifdef LOG
@@ -939,15 +943,15 @@ static void decode_audio_header (demux_ogg_t *this, const int stream_num, ogg_pa
     int32_t          locsize, locdefault_len, locbuffersize, locavgbytespersec;
     int64_t          loctime_unit, locsamples_per_unit;
 
-    locsize = LE_32(&op->packet[13]);
-    loctime_unit = LE_64(&op->packet[17]);
-    locsamples_per_unit = LE_64(&op->packet[25]);
-    locdefault_len = LE_32(&op->packet[33]);
-    locbuffersize = LE_32(&op->packet[37]);
-    locbits_per_sample = LE_16(&op->packet[41]);
-    locchannels = LE_16(&op->packet[45]);
-    locblockalign = LE_16(&op->packet[47]);
-    locavgbytespersec= LE_32(&op->packet[49]);
+    locsize = _X_LE_32(&op->packet[13]);
+    loctime_unit = _X_LE_64(&op->packet[17]);
+    locsamples_per_unit = _X_LE_64(&op->packet[25]);
+    locdefault_len = _X_LE_32(&op->packet[33]);
+    locbuffersize = _X_LE_32(&op->packet[37]);
+    locbits_per_sample = _X_LE_16(&op->packet[41]);
+    locchannels = _X_LE_16(&op->packet[45]);
+    locblockalign = _X_LE_16(&op->packet[47]);
+    locavgbytespersec= _X_LE_32(&op->packet[49]);
 
     lprintf ("direct show filter created audio stream detected, hexdump:\n");
 #ifdef LOG
@@ -1016,7 +1020,7 @@ static void decode_dshow_header (demux_ogg_t *this, const int stream_num, ogg_pa
 
   this->si[stream_num]->headers = 0; /* header is sent below */
 
-  if ( (LE_32(&op->packet[96]) == 0x05589f80) && (op->bytes >= 184)) {
+  if ( (_X_LE_32(&op->packet[96]) == 0x05589f80) && (op->bytes >= 184)) {
 
     buf_element_t    *buf;
     xine_bmiheader    bih;
@@ -1035,11 +1039,11 @@ static void decode_dshow_header (demux_ogg_t *this, const int stream_num, ogg_pa
     this->si[stream_num]->buf_types |= channel;
 
     bih.biSize          = sizeof(xine_bmiheader);
-    bih.biWidth         = LE_32(&op->packet[176]);
-    bih.biHeight        = LE_32(&op->packet[180]);
+    bih.biWidth         = _X_LE_32(&op->packet[176]);
+    bih.biHeight        = _X_LE_32(&op->packet[180]);
     bih.biPlanes        = 0;
     memcpy (&bih.biCompression, op->packet+68, 4);
-    bih.biBitCount      = LE_16(&op->packet[182]);
+    bih.biBitCount      = _X_LE_16(&op->packet[182]);
     if (!bih.biBitCount)
       bih.biBitCount = 24; /* FIXME ? */
     bih.biSizeImage     = (bih.biBitCount>>3)*bih.biWidth*bih.biHeight;
@@ -1075,7 +1079,7 @@ static void decode_dshow_header (demux_ogg_t *this, const int stream_num, ogg_pa
 
     this->ignore_keyframes = 1;
 
-  } else if (LE_32(&op->packet[96]) == 0x05589F81) {
+  } else if (_X_LE_32(&op->packet[96]) == 0x05589F81) {
 
 #if 0
     /* FIXME: no test streams */
@@ -1190,7 +1194,7 @@ static void decode_theora_header (demux_ogg_t *this, const int stream_num, ogg_p
 
 static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_packet *op) {
   xine_flac_metadata_header header;
-  xine_flac_streaminfo_block streaminfo;
+  xine_flac_streaminfo_block streaminfo = {};
   buf_element_t *buf;
   xine_waveformatex wave;
 
@@ -1205,7 +1209,7 @@ static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_pac
   _x_assert(op->packet[5] == 1); _x_assert(op->packet[6] == 0);
 
   /* Header count */
-  this->si[stream_num]->headers = 0/*BE_16(&op->packet[7]) +1*/;
+  this->si[stream_num]->headers = 0/*_X_BE_16(&op->packet[7]) +1*/;
 
   /* fLaC signature */
   _x_assert(op->packet[9] == 'f'); _x_assert(op->packet[10] == 'L');
@@ -1217,16 +1221,17 @@ static void decode_flac_header (demux_ogg_t *this, const int stream_num, ogg_pac
   case FLAC_BLOCKTYPE_STREAMINFO:
     _x_assert(header.length == FLAC_STREAMINFO_SIZE);
     _x_parse_flac_streaminfo_block(&op->packet[17], &streaminfo);
+
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, streaminfo.samplerate);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, streaminfo.channels);
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS, streaminfo.bits_per_sample);
+
     break;
   }
 
   this->si[stream_num]->buf_types = BUF_AUDIO_FLAC
     +this->num_audio_streams++;
 
-  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE, streaminfo.samplerate);
-  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_CHANNELS, streaminfo.channels);
-  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITS, streaminfo.bits_per_sample);
-
   this->si[stream_num]->factor = 90000;
 
   buf = this->audio_fifo->buffer_pool_alloc(this->audio_fifo);
@@ -1268,9 +1273,9 @@ static void decode_anxdata_header (demux_ogg_t *this, const int stream_num, ogg_
   lprintf("AnxData stream detected\n");
 
   /* read granule rate */
-  granule_rate_n = LE_64(&op->packet[8]);
-  granule_rate_d = LE_64(&op->packet[16]);
-  secondary_headers = LE_32(&op->packet[24]);
+  granule_rate_n = _X_LE_64(&op->packet[8]);
+  granule_rate_d = _X_LE_64(&op->packet[16]);
+  secondary_headers = _X_LE_32(&op->packet[24]);
 
   lprintf("granule_rate %" PRId64 "/%" PRId64 ", %d secondary headers\n",
       granule_rate_n, granule_rate_d, secondary_headers);
@@ -1326,8 +1331,8 @@ static void decode_cmml_header (demux_ogg_t *this, const int stream_num, ogg_pac
     this->si[stream_num]->headers = 0;
     this->si[stream_num]->buf_types = BUF_SPU_CMML | channel;
 
-    this->si[stream_num]->factor = 90000 * LE_64(&op->packet[20]);
-    this->si[stream_num]->quotient = LE_64(&op->packet[12]);
+    this->si[stream_num]->factor = 90000 * _X_LE_64(&op->packet[20]);
+    this->si[stream_num]->quotient = _X_LE_64(&op->packet[12]);
     this->si[stream_num]->granuleshift = (int)op->packet[28];
 }
 
@@ -1348,7 +1353,6 @@ static void send_header (demux_ogg_t *this) {
 
   while (!done) {
     if (!read_ogg_packet(this)) {
-      this->status = DEMUX_FINISHED;
       return;
     }
     /* now we've got at least one new page */
@@ -1865,9 +1869,7 @@ static int format_lang_string (demux_ogg_t * this, uint32_t buf_mask, uint32_t b
   for (stream_num=0; stream_num<this->num_streams; stream_num++) {
     if ((this->si[stream_num]->buf_types & buf_mask) == buf_type) {
       if (this->si[stream_num]->language) {
-        strncpy (str, this->si[stream_num]->language, XINE_LANG_MAX);
-        str[XINE_LANG_MAX - 1] = '\0';
-        if (strlen(this->si[stream_num]->language) >= XINE_LANG_MAX)
+        if (snprintf (str, XINE_LANG_MAX, "%s", this->si[stream_num]->language) >= XINE_LANG_MAX)
           /* the string got truncated */
           str[XINE_LANG_MAX - 2] = str[XINE_LANG_MAX - 3] = str[XINE_LANG_MAX - 4] = '.';
         /* TODO: provide long version in XINE_META_INFO_FULL_LANG */
@@ -2116,7 +2118,12 @@ static const char *anx_get_extensions (demux_class_t *this_gen) {
 }
 
 static const char *anx_get_mimetypes (demux_class_t *this_gen) {
-  return "application/x-annodex: ogg: Annodex media;";
+  return "application/annodex: anx: Annodex media;"
+         "application/x-annodex: anx: Annodex media;"
+         "audio/annodex: axa: Annodex audio;"
+         "audio/x-annodex: axa: Annodex audio;"
+         "video/annodex: axv: Annodex video;"
+         "video/x-annodex: axv: Annodex video;";
 }
 
 static void anx_class_dispose (demux_class_t *this_gen) {
@@ -2153,14 +2160,16 @@ static const char *ogg_get_identifier (demux_class_t *this_gen) {
 }
 
 static const char *ogg_get_extensions (demux_class_t *this_gen) {
-  return "ogg ogm spx";
+  return "ogx ogv oga ogg spx ogm";
 }
 
 static const char *ogg_get_mimetypes (demux_class_t *this_gen) {
-  return "audio/x-ogg: ogg: OggVorbis Audio;"
-         "audio/x-speex: ogg: Speex Audio;"
-         "application/x-ogg: ogg: Ogg Stream;"
-         "application/ogg: ogg: Ogg Stream;";
+  return "application/ogg: ogx: Ogg Stream;"
+         "application/x-ogg: ogx: Ogg Stream;"
+         "audio/ogg: oga: Ogg Audio;"
+         "audio/x-ogg: oga: Ogg Audio;"
+         "video/ogg: ogv: Ogg Video;"
+         "video/x-ogg: ogv: Ogg Video;";
 }
 
 static void ogg_class_dispose (demux_class_t *this_gen) {
