@@ -15,15 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
  * RoQ File Demuxer by Mike Melanson (melanson@pcisys.net)
  * For more information regarding the RoQ file format, visit:
  *   http://www.csse.monash.edu.au/~timf/
- *
- * $Id: demux_roq.c,v 1.53 2004/06/13 21:28:54 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -95,8 +93,8 @@ static int open_roq_file(demux_roq_t *this) {
     return 0;
 
   /* check for the RoQ magic numbers */
-  if ((LE_16(&preamble[0]) != RoQ_MAGIC_NUMBER) ||
-      (LE_32(&preamble[2]) != 0xFFFFFFFF))
+  if ((_X_LE_16(&preamble[0]) != RoQ_MAGIC_NUMBER) ||
+      (_X_LE_32(&preamble[2]) != 0xFFFFFFFF))
     return 0;
     
   this->bih.biSize = sizeof(xine_bmiheader);
@@ -112,7 +110,7 @@ static int open_roq_file(demux_roq_t *this) {
    *
    * therefore, the frame pts increment is 90000 / fps
    */
-  fps = LE_16(&preamble[6]);
+  fps = _X_LE_16(&preamble[6]);
   this->frame_pts_inc = 90000 / fps;
 
   /* iterate through the first 2 seconds worth of chunks searching for
@@ -124,16 +122,16 @@ static int open_roq_file(demux_roq_t *this) {
     if (this->input->read(this->input, preamble, RoQ_CHUNK_PREAMBLE_SIZE) != 
       RoQ_CHUNK_PREAMBLE_SIZE)
       break;
-    chunk_type = LE_16(&preamble[0]);
-    chunk_size = LE_32(&preamble[2]);
+    chunk_type = _X_LE_16(&preamble[0]);
+    chunk_size = _X_LE_32(&preamble[2]);
 
     if (chunk_type == RoQ_INFO) {
       /* fetch the width and height; reuse the preamble bytes */
       if (this->input->read(this->input, preamble, 8) != 8)
         break;
 
-      this->bih.biWidth = LE_16(&preamble[0]);
-      this->bih.biHeight = LE_16(&preamble[2]);
+      this->bih.biWidth = _X_LE_16(&preamble[0]);
+      this->bih.biHeight = _X_LE_16(&preamble[2]);
 
       /* if an audio chunk was already found, search is done */
       if (this->wave.nChannels)
@@ -188,8 +186,8 @@ static int demux_roq_send_chunk(demux_plugin_t *this_gen) {
     this->status = DEMUX_FINISHED;
     return this->status;
   }
-  chunk_type = LE_16(&preamble[0]);
-  chunk_size = LE_32(&preamble[2]);
+  chunk_type = _X_LE_16(&preamble[0]);
+  chunk_size = _X_LE_32(&preamble[2]);
 
   /* if the chunk is an audio chunk, route it to the audio fifo */
   if ((chunk_type == RoQ_SOUND_MONO) || (chunk_type == RoQ_SOUND_STEREO)) {
@@ -432,7 +430,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   switch (stream->content_detection_method) {
 
   case METHOD_BY_EXTENSION: {
-    char *extensions, *mrl;
+    const char *extensions, *mrl;
 
     mrl = input->get_mrl (input);
     extensions = class_gen->get_extensions (class_gen);
@@ -462,19 +460,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static char *get_description (demux_class_t *this_gen) {
+static const char *get_description (demux_class_t *this_gen) {
   return "Id RoQ file demux plugin";
 }
 
-static char *get_identifier (demux_class_t *this_gen) {
+static const char *get_identifier (demux_class_t *this_gen) {
   return "RoQ";
 }
 
-static char *get_extensions (demux_class_t *this_gen) {
+static const char *get_extensions (demux_class_t *this_gen) {
   return "roq";
 }
 
-static char *get_mimetypes (demux_class_t *this_gen) {
+static const char *get_mimetypes (demux_class_t *this_gen) {
   return NULL;
 }
 
