@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * ID3 tag parser
  *
@@ -28,8 +28,6 @@
  *          unzip support
  *
  * ID3v2 specs: http://www.id3.org/
- *
- * $Id: id3.c,v 1.11 2005/09/16 23:01:10 tmattern Exp $
  */
  
 #ifdef HAVE_CONFIG_H
@@ -47,7 +45,7 @@
 #include "bswap.h"
 #include "id3.h"
 
-#define ID3_GENRE_COUNT 126
+#define ID3_GENRE_COUNT (sizeof (id3_genre) / sizeof (id3_genre[0]))
 static const char* const id3_genre[] =
   {"Blues", "Classic Rock", "Country", "Dance", "Disco",
    "Funk", "Grunge", "Hip-Hop", "Jazz", "Metal",
@@ -74,7 +72,11 @@ static const char* const id3_genre[] =
    "Satire", "Slow Jam", "Club", "Tango", "Samba",
    "Folklore", "Ballad", "Power Ballad", "Rhythmic Soul", "Freestyle",
    "Duet", "Punk Rock", "Drum Solo", "A capella", "Euro-House",
-   "Dance Hall" };
+   "Dance Hall", "Goa", "Drum & Bass", "Club-House", "Hardcore", "Terror",
+   "Indie", "BritPop", "Negerpunk", "Polsk Punk", "Beat",
+   "Christian Gangsta Rap", "Heavy Metal", "Black Metal", "Crossover",
+   "Contemporary Christian", "Christian Rock", "Merengue", "Salsa",
+   "Thrash Metal", "Anime", "JPop", "Synthpop" };
 
 #define ID3_ENCODING_COUNT 4
 static const char* const id3_encoding[] = {
@@ -227,45 +229,15 @@ static int id3v2_parse_genre(char* dest, char *src, int len) {
   return 1;
 }
 
-/* parse an unsynchronized 16bits integer */
-static uint16_t BE_16_synchsafe(uint8_t buf[2]) {
-  return ((uint16_t)(buf[0] & 0x7F) << 7) |
-          (uint16_t)(buf[1] & 0x7F);
-}
-
-/* parse an unsynchronized 24bits integer */
-static uint32_t BE_24_synchsafe(uint8_t buf[3]) {
-  return ((uint32_t)(buf[0] & 0x7F) << 14) |
-         ((uint32_t)(buf[1] & 0x7F) << 7) |
-          (uint32_t)(buf[2] & 0x7F);
-}
-
-/* parse an unsynchronized 32bits integer */
-static uint32_t BE_32_synchsafe(uint8_t buf[4]) {
-  return ((uint32_t)(buf[0] & 0x7F) << 21) |
-         ((uint32_t)(buf[1] & 0x7F) << 14) |
-         ((uint32_t)(buf[2] & 0x7F) << 7) |
-          (uint32_t)(buf[3] & 0x7F);
-}
-
-/* parse an unsynchronized 35bits integer */
-static uint32_t BE_35_synchsafe(uint8_t buf[5]) {
-  return ((uint32_t)(buf[0] & 0x07) << 28) |
-         ((uint32_t)(buf[1] & 0x7F) << 21) |
-         ((uint32_t)(buf[2] & 0x7F) << 14) |
-         ((uint32_t)(buf[3] & 0x7F) << 7) |
-          (uint32_t)(buf[4] & 0x7F);
-}
-
 static int id3v2_parse_header(input_plugin_t *input, uint8_t *mp3_frame_header,
                               id3v2_header_t *tag_header) {
   uint8_t buf[6];
 
-  tag_header->id = BE_32(mp3_frame_header);
+  tag_header->id = _X_BE_32(mp3_frame_header);
   if (input->read (input, buf, 6) == 6) {
     tag_header->revision = buf[0];
     tag_header->flags    = buf[1];
-    tag_header->size     = BE_32_synchsafe(&buf[2]);
+    tag_header->size     = _X_BE_32_synchsafe(&buf[2]);
 
     lprintf("tag: ID3 v2.%d.%d\n", mp3_frame_header[3], tag_header->revision);
     lprintf("flags: %d\n", tag_header->flags);
@@ -287,7 +259,7 @@ static int id3v22_parse_frame_header(input_plugin_t *input,
   if (len == ID3V22_FRAME_HEADER_SIZE) {
     frame_header->id   = (buf[0] << 16) + (buf[1] << 8) + buf[2];
 
-    frame_header->size = BE_24_synchsafe(&buf[3]);
+    frame_header->size = _X_BE_24_synchsafe(&buf[3]);
 
     lprintf("frame: %c%c%c: size: %d\n", buf[0], buf[1], buf[2],
             frame_header->size);
@@ -377,21 +349,21 @@ int id3v22_parse_tag(input_plugin_t *input,
     if (tag_header.flags & ID3V22_ZERO_FLAG) {
       /* invalid flags */
       xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-              "id3: invalid header flags\n");
+              LOG_MODULE ": invalid header flags (%02x)\n", tag_header.flags);
       input->seek (input, tag_header.size - pos, SEEK_CUR);
       return 0;
     }
     if (tag_header.flags & ID3V22_COMPRESS_FLAG) {
       /* compressed tag: not supported */
       xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-              "id3: compressed tags are not supported\n");
+              LOG_MODULE ": compressed tags are not supported\n");
       input->seek (input, tag_header.size - pos, SEEK_CUR);
       return 0;
     }
     if (tag_header.flags & ID3V22_UNSYNCH_FLAG) {
       /* unsynchronized tag: not supported */
       xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-              "id3: unsynchronized tags are not supported\n");
+              LOG_MODULE ": unsynchronized tags are not supported\n");
       input->seek (input, tag_header.size - pos, SEEK_CUR);
       return 0;
     }
@@ -403,11 +375,11 @@ int id3v22_parse_tag(input_plugin_t *input,
           if ((pos + tag_frame_header.size) <= tag_header.size) {
             if (!id3v22_interp_frame(input, stream, &tag_frame_header)) {
               xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                      "id3: invalid frame content\n");
+                      LOG_MODULE ": invalid frame content\n");
             }
           } else {
             xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                    "id3: invalid frame header\n");
+                    LOG_MODULE ": invalid frame header\n");
             input->seek (input, tag_header.size - pos, SEEK_CUR);
             return 1;
           }
@@ -419,13 +391,13 @@ int id3v22_parse_tag(input_plugin_t *input,
         }
       } else {
         xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                "id3: id3v2_parse_frame_header problem\n");
+                LOG_MODULE ": id3v2_parse_frame_header problem\n");
         return 0;
       }
     }
     return 1;
   } else {
-    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, "id3: id3v2_parse_header problem\n");
+    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": id3v2_parse_header problem\n");
     return 0;
   }
 }
@@ -439,9 +411,9 @@ static int id3v23_parse_frame_header(input_plugin_t *input,
 
   len  = input->read (input, buf, ID3V23_FRAME_HEADER_SIZE);
   if (len == ID3V23_FRAME_HEADER_SIZE) {
-    frame_header->id    = BE_32(buf);
-    frame_header->size  = BE_32(&buf[4]);
-    frame_header->flags = BE_16(buf + 8);
+    frame_header->id    = _X_BE_32(buf);
+    frame_header->size  = _X_BE_32(&buf[4]);
+    frame_header->flags = _X_BE_16(buf + 8);
 
     lprintf("frame: %c%c%c%c, size: %d, flags: %X\n", buf[0], buf[1], buf[2], buf[3],
             frame_header->size, frame_header->flags);
@@ -458,12 +430,12 @@ static int id3v23_parse_frame_ext_header(input_plugin_t *input,
 
   if (input->read (input, buf, 4) == 4) {
   
-    frame_ext_header->size  = BE_32_synchsafe(&buf[0]);
+    frame_ext_header->size  = _X_BE_32_synchsafe(&buf[0]);
     
     if (frame_ext_header->size == 6) {
       if (input->read (input, buf + 4, 6) == 6) {
-        frame_ext_header->flags = BE_16(buf + 4);
-        frame_ext_header->padding_size = BE_32(buf + 6);
+        frame_ext_header->flags = _X_BE_16(buf + 4);
+        frame_ext_header->padding_size = _X_BE_32(buf + 6);
         frame_ext_header->crc = 0;
       } else {
         return 0;
@@ -471,9 +443,9 @@ static int id3v23_parse_frame_ext_header(input_plugin_t *input,
 
     } else if (frame_ext_header->size == 10) {
       if (input->read (input, buf + 4, 10) == 10) {
-        frame_ext_header->flags = BE_16(buf + 4);
-        frame_ext_header->padding_size = BE_32(buf + 6);
-        frame_ext_header->crc = BE_32(buf + 10);
+        frame_ext_header->flags = _X_BE_16(buf + 4);
+        frame_ext_header->padding_size = _X_BE_32(buf + 6);
+        frame_ext_header->crc = _X_BE_32(buf + 10);
       } else {
         return 0;
       }
@@ -571,14 +543,14 @@ int id3v23_parse_tag(input_plugin_t *input,
     if (tag_header.flags & ID3V23_ZERO_FLAG) {
       /* invalid flags */
       xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-              "id3: invalid header flags\n");
+              LOG_MODULE ": invalid header flags (%02x)\n", tag_header.flags);
       input->seek (input, tag_header.size - pos, SEEK_CUR);
       return 0;
     }
     if (tag_header.flags & ID3V23_UNSYNCH_FLAG) {
       /* unsynchronized tag: not supported */
       xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-              "id3: unsynchronized tags are not supported\n");
+              LOG_MODULE ": unsynchronized tags are not supported\n");
       input->seek (input, tag_header.size - pos, SEEK_CUR);
       return 0;
     }
@@ -587,6 +559,7 @@ int id3v23_parse_tag(input_plugin_t *input,
       if (!id3v23_parse_frame_ext_header(input, &tag_frame_ext_header)) {
         return 0;
       }
+      pos += tag_frame_ext_header.size;
     }
     /* frame parsing */
     while ((pos + ID3V23_FRAME_HEADER_SIZE) <= tag_header.size) {
@@ -596,29 +569,30 @@ int id3v23_parse_tag(input_plugin_t *input,
           if ((pos + tag_frame_header.size) <= tag_header.size) {
             if (!id3v23_interp_frame(input, stream, &tag_frame_header)) {
               xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                      "id3: invalid frame content\n");
+                      LOG_MODULE ": invalid frame content\n");
             }
           } else {
             xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                    "id3: invalid frame header\n");
+                    LOG_MODULE ": invalid frame header\n");
             input->seek (input, tag_header.size - pos, SEEK_CUR);
             return 1;
           }
           pos += tag_frame_header.size;
         } else {
           /* end of frames, the rest is padding */
-          input->seek (input, tag_header.size - pos, SEEK_CUR);
+	  lprintf("skipping padding %d bytes\n", tag_header.size - pos);
+	  input->seek (input, tag_header.size - pos, SEEK_CUR);
           return 1;
         }
       } else {
         xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                "id3: id3v2_parse_frame_header problem\n");
+                LOG_MODULE ": id3v2_parse_frame_header problem\n");
         return 0;
       }
     }
     return 1;
   } else {
-    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, "id3v23: id3v2_parse_header problem\n");
+    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": id3v2_parse_header problem\n");
     return 0;
   }
 }
@@ -627,15 +601,15 @@ int id3v23_parse_tag(input_plugin_t *input,
 
 /* id3v2 "genre" parsing code. what a ugly format ! */
 static int id3v24_parse_genre(char* dest, char *src, int len) {
-  int index = 0;
+  unsigned int index = 0;
   
   dest[0] = '\0';
-  if (sscanf(src, "%2d", &index) == 1) {
+  if (sscanf(src, "%u", &index) == 1) {
     if (index < ID3_GENRE_COUNT) {
       strncpy(dest, id3_genre[index], len);
       dest[len - 1] = '\0';
     } else {
-      lprintf("invalid index: %d\n", index);
+      lprintf("invalid index: %u\n", index);
     }
   }
   return 1;
@@ -648,9 +622,9 @@ static int id3v24_parse_frame_header(input_plugin_t *input,
 
   len  = input->read (input, buf, ID3V24_FRAME_HEADER_SIZE);
   if (len == ID3V24_FRAME_HEADER_SIZE) {
-    frame_header->id    = BE_32(buf);
-    frame_header->size  = BE_32_synchsafe(&buf[4]);
-    frame_header->flags = BE_16(&buf[8]);
+    frame_header->id    = _X_BE_32(buf);
+    frame_header->size  = _X_BE_32_synchsafe(&buf[4]);
+    frame_header->flags = _X_BE_16(&buf[8]);
 
     lprintf("frame: %c%c%c%c, size: %d, flags: %X\n", buf[0], buf[1], buf[2], buf[3],
             frame_header->size, frame_header->flags);
@@ -667,7 +641,7 @@ static int id3v24_parse_ext_header(input_plugin_t *input,
 
   if (input->read (input, buf, 4) == 4) {
  
-    frame_ext_header->size  = BE_32_synchsafe(&buf[0]);
+    frame_ext_header->size  = _X_BE_32_synchsafe(&buf[0]);
 
     if (input->read (input, buf, 2) == 2) {
       uint8_t flags_size = buf[0];
@@ -822,7 +796,7 @@ int id3v24_parse_tag(input_plugin_t *input,
     if (tag_header.flags & ID3V24_ZERO_FLAG) {
       /* invalid flags */
       xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-              "id3: invalid header flags\n");
+              LOG_MODULE ": invalid header flags (%02x)\n", tag_header.flags);
       input->seek (input, tag_header.size - pos, SEEK_CUR);
       return 0;
     }
@@ -837,6 +811,7 @@ int id3v24_parse_tag(input_plugin_t *input,
       if (!id3v24_parse_ext_header(input, &tag_frame_ext_header)) {
         return 0;
       }
+      pos += tag_frame_ext_header.size;
     }
     /* frame parsing */
     while ((pos + ID3V24_FRAME_HEADER_SIZE) <= tag_header.size) {
@@ -846,11 +821,11 @@ int id3v24_parse_tag(input_plugin_t *input,
           if ((pos + tag_frame_header.size) <= tag_header.size) {
             if (!id3v24_interp_frame(input, stream, &tag_frame_header)) {
               xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                      "id3: invalid frame content\n");
+                      LOG_MODULE ": invalid frame content\n");
             }
           } else {
             xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                    "id3: invalid frame header\n");
+                    LOG_MODULE ": invalid frame header\n");
             input->seek (input, tag_header.size - pos, SEEK_CUR);
             return 1;
           }
@@ -862,7 +837,7 @@ int id3v24_parse_tag(input_plugin_t *input,
         }
       } else {
         xprintf(stream->xine, XINE_VERBOSITY_DEBUG, 
-                "id3: id3v2_parse_frame_header problem\n");
+                LOG_MODULE ": id3v2_parse_frame_header problem\n");
         return 0;
       }
     }
@@ -872,7 +847,37 @@ int id3v24_parse_tag(input_plugin_t *input,
     }
     return 1;
   } else {
-    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, "id3v23: id3v2_parse_header problem\n");
+    xprintf(stream->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": id3v2_parse_header problem\n");
     return 0;
   }
 }
+
+int id3v2_parse_tag(input_plugin_t *input,
+		    xine_stream_t *stream,
+		    int8_t *mp3_frame_header) {
+  _x_assert(mp3_frame_header[0] == 'I' && mp3_frame_header[1] == 'D' && mp3_frame_header[2] == '3');
+
+  int result = 0;
+
+  switch(mp3_frame_header[3]) {
+  case 2:
+    xprintf(stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": ID3V2.2 tag\n");
+    result = id3v22_parse_tag(input, stream, mp3_frame_header);
+    break;
+   
+  case 3:
+    xprintf(stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": ID3V2.3 tag\n");
+    result = id3v23_parse_tag(input, stream, mp3_frame_header);
+    break;
+
+  case 4:
+    xprintf(stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": ID3V2.4 tag\n");
+    result = id3v24_parse_tag(input, stream, mp3_frame_header);
+    break;
+
+  default:
+    xprintf(stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE ": Unknown ID3v2 version: 0x%02x.\n", mp3_frame_header[3]);
+  }
+
+  return result;
+}
