@@ -15,11 +15,9 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * input plugin for http network streams
- *
- * $Id: input_http.c,v 1.127 2007/01/19 01:05:25 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -64,6 +62,7 @@
 #define TAG_ICY_NOTICE2    "icy-notice2:"
 #define TAG_ICY_METAINT    "icy-metaint:"
 #define TAG_CONTENT_TYPE   "Content-Type:"
+#define TAG_LASTFM_SERVER  "Server: last.fm "
 
 typedef struct {
   input_plugin_t   input_plugin;
@@ -94,6 +93,9 @@ typedef struct {
   char             preview[MAX_PREVIEW_SIZE];
   off_t            preview_size;
   
+  /* Last.FM streaming server */
+  unsigned char    is_lastfm;
+
   /* ShoutCast */
   int              shoutcast_mode;
   int              shoutcast_metaint;
@@ -181,7 +183,7 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
     /* \177\0\0\1 is the *octal* representation of 127.0.0.1 */
     if ( info->h_addrtype == AF_INET && !memcmp(info->h_addr_list[0], "\177\0\0\1", 4) ) {
       lprintf("host '%s' is localhost\n", host);
-      return 1;
+      return 0;
     }
     /* TODO: IPv6 check */
   }
@@ -200,13 +202,13 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
       /* special-case domain beginning with '=' -> is a host name */
       if (domain[0] == '=' && strcmp(target, domain + 1) == 0) {
 	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
-	return 1;
+	return 0;
       }
       noprox_len = strlen(domain);
       /* special-case host==domain, avoiding dot checks */
       if (host_len == noprox_len && strcmp(target, domain) == 0) {
 	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
-	return 1;
+	return 0;
       }
       /* check for host in domain, and require that (if matched) the domain
        * name is preceded by a dot, either in the host or domain strings,
@@ -216,7 +218,7 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
 	  && (domain[0] == '.' || target[host_len - noprox_len - 1] == '.')
 	  && strcmp(target + host_len - noprox_len, domain) == 0) {
 	lprintf("host '%s' is in no-proxy domain '%s'\n", target, domain);
-	return 1;
+	return 0;
       }
       lprintf("host '%s' isn't in no-proxy domain '%s'\n", target, domain);
     }
@@ -225,7 +227,7 @@ static int _x_use_proxy(http_input_class_t *this, const char *host) {
   }
   free(no_proxy);
 
-  return 0;
+  return 1;
 }
 
 static int http_plugin_basicauth (const char *user, const char *password, char* dest, int len) {
@@ -363,7 +365,7 @@ static off_t http_plugin_read_int (http_input_plugin_t *this,
   int read_bytes = 0;
   int nlen;
   
-  lprintf("total=%lld\n", total);
+  lprintf("total=%"PRId64"\n", total);
   while (total) {
     nlen = total;
     if (this->shoutcast_mode &&
@@ -383,6 +385,25 @@ static off_t http_plugin_read_int (http_input_plugin_t *this,
       if (nlen < 0)
         goto error;
 
+      /* Identify SYNC string for last.fm, this is limited to last.fm
+       * streaming servers to avoid hitting on tracks metadata for other
+       * servers.
+       */
+      if ( this->is_lastfm &&
+	   memmem(&buf[read_bytes], nlen, "SYNC", 4) != NULL ) {
+	/* Tell frontend to update the UI */
+	const xine_event_t event = {
+	  .type = XINE_EVENT_UI_CHANNELS_CHANGED,
+	  .stream = this->stream,
+	  .data = NULL,
+	  .data_length = 0
+	};
+
+	lprintf("SYNC from last.fm server received\n");
+
+	xine_event_send(this->stream, &event);
+      }
+
       this->shoutcast_pos += nlen;
     }
     
@@ -415,7 +436,7 @@ static off_t http_plugin_read (input_plugin_t *this_gen,
     else
       n = nlen;
 
-    lprintf ("%lld bytes from preview (which has %lld bytes)\n", n, this->preview_size);
+    lprintf ("%"PRId64" bytes from preview (which has %"PRId64" bytes)\n", n, this->preview_size);
     memcpy (buf, &this->preview[this->curpos], n);
 
     num_bytes += n;
@@ -643,6 +664,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
   int                  buflen;
   int                  use_proxy;
   int                  proxyport;
+  int                  mpegurl_redirect = 0;
   
   use_proxy = this_class->proxyhost && strlen(this_class->proxyhost);
   
@@ -663,9 +685,8 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
     _x_message(this->stream, XINE_MSG_GENERAL_WARNING, "malformed url", NULL);
     return 0;
   }
-  if (use_proxy && _x_use_proxy(this_class, this->host)) {
-    use_proxy = 0;
-  }
+  use_proxy = use_proxy && _x_use_proxy(this_class, this->host);
+
   if (this->port == 0)
     this->port = DEFAULT_HTTP_PORT;
   
@@ -804,18 +825,20 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 
       if (linenum == 1) {
         int httpver, httpsub;
-	char httpstatus[51];
+	char httpstatus[51] = { 0, };
 
-	if (sscanf(this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
-		   &httpcode, httpstatus) != 4)	{
-	  
-	  /* icecast 1 ? */
-	  if (sscanf(this->buf, "ICY %d %50[^\015\012]", &httpcode, httpstatus) != 2)	{
+	if (
+	    (sscanf(this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
+		    &httpcode, httpstatus) != 4) &&
+	    (sscanf(this->buf, "HTTP/%d.%d %d", &httpver, &httpsub,
+		    &httpcode) != 3) &&
+	    (sscanf(this->buf, "ICY %d %50[^\015\012]", /* icecast 1 ? */
+		    &httpcode, httpstatus) != 2)
+	   ) {
 	    _x_message(this->stream, XINE_MSG_CONNECTION_REFUSED, "invalid http answer", NULL);
 	    xine_log (this->stream->xine, XINE_LOG_MSG, 
 		      _("input_http: invalid http answer\n"));
 	    return -6;
-	  }
 	}
 
 	if (httpcode >= 300 && httpcode < 400) {
@@ -828,7 +851,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 		    _("input_http: http status not 2xx: >%d %s<\n"),
 		                        httpcode, httpstatus);
 	  return -7;
-	} else if (httpcode == 403) {
+	} else if (httpcode == 403 || httpcode == 401) {
           _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
 	  xine_log (this->stream->xine, XINE_LOG_MSG,
 		    _("input_http: http status not 2xx: >%d %s<\n"),
@@ -865,6 +888,16 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
           return http_plugin_open(this_gen);
         }
 
+	{
+	  static const char mpegurl_ct_str[] = "Content-Type: audio/x-mpegurl";
+	  static const size_t mpegurl_ct_size = sizeof(mpegurl_ct_str)-1;
+	  if (!strncasecmp(this->buf, mpegurl_ct_str, mpegurl_ct_size)) {
+	    lprintf("Opening an audio/x-mpegurl file, late redirect.");
+
+	    mpegurl_redirect = 1;
+	  }
+	}
+
         /* Icecast / ShoutCast Stuff */
         if (!strncasecmp(this->buf, TAG_ICY_NAME, sizeof(TAG_ICY_NAME) - 1)) {
           _x_meta_info_set(this->stream, XINE_META_INFO_ALBUM,
@@ -906,6 +939,10 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
             this->is_nsv = 1;
           }
         }
+	if ( !strncasecmp(this->buf, TAG_LASTFM_SERVER, sizeof(TAG_LASTFM_SERVER)-1) ) {
+	  lprintf("last.fm streaming server detected\n");
+	  this->is_lastfm = 1;
+	}
       }
  
       if (len == -1)
@@ -924,6 +961,30 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
 
   lprintf ("end of headers\n");
 
+  if ( mpegurl_redirect ) {
+    char buf[4096] = { 0, };
+    char *newline = NULL;
+
+    http_plugin_read_int(this, buf, 4095);
+    newline = strstr(buf, "\r\n");
+
+    /* If the newline can't be found, either the 4K buffer is too small, or
+     * more likely something is fuzzy.
+     */
+    if ( newline ) {
+      char *href;
+
+      *newline = '\0';
+
+      lprintf("mpegurl pointing to %s\n", buf);
+      
+      href = _x_canonicalise_url (this->mrl, buf);
+      free(this->mrl);
+      this->mrl = href;
+      return http_plugin_open(this_gen);
+    }
+  }
+
   /*
    * fill preview buffer
    */
@@ -942,7 +1003,7 @@ static int http_plugin_open (input_plugin_t *this_gen ) {
     return -12;
   }
   
-  lprintf("preview_size=%lld\n", this->preview_size);
+  lprintf("preview_size=%"PRId64"\n", this->preview_size);
   this->curpos = 0;
   
   return 1;
