@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: mmsh.c,v 1.34 2005/01/12 00:05:38 tmattern Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * MMS over HTTP protocol
  *   written by Thibaut Mattern
@@ -82,61 +80,61 @@
 
 #define SCRATCH_SIZE             1024
 
-static const char* mmsh_FirstRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Pragma: no-cache,rate=1.000000,stream-time=0,stream-offset=0:0,request-context=%u,max-duration=0\r\n"
-    CLIENTGUID
-    "Connection: Close\r\n\r\n";
+#define mmsh_FirstRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Pragma: no-cache,rate=1.000000,stream-time=0,stream-offset=0:0,request-context=%u,max-duration=0\r\n" \
+    CLIENTGUID \
+    "Connection: Close\r\n\r\n"
 
-static const char* mmsh_SeekableRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Pragma: no-cache,rate=1.000000,stream-time=%u,stream-offset=%u:%u,request-context=%u,max-duration=%u\r\n"
-    CLIENTGUID
-    "Pragma: xPlayStrm=1\r\n"
-    "Pragma: stream-switch-count=%d\r\n"
-    "Pragma: stream-switch-entry=%s\r\n" /*  ffff:1:0 ffff:2:0 */
-    "Connection: Close\r\n\r\n";
+#define mmsh_SeekableRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Pragma: no-cache,rate=1.000000,stream-time=%u,stream-offset=%u:%u,request-context=%u,max-duration=%u\r\n" \
+    CLIENTGUID \
+    "Pragma: xPlayStrm=1\r\n" \
+    "Pragma: stream-switch-count=%d\r\n" \
+    "Pragma: stream-switch-entry=%s\r\n" /*  ffff:1:0 ffff:2:0 */ \
+    "Connection: Close\r\n\r\n"
 
-static const char* mmsh_LiveRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Pragma: no-cache,rate=1.000000,request-context=%u\r\n"
-    "Pragma: xPlayStrm=1\r\n"
-    CLIENTGUID
-    "Pragma: stream-switch-count=%d\r\n"
-    "Pragma: stream-switch-entry=%s\r\n"
-    "Connection: Close\r\n\r\n";
+#define mmsh_LiveRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Pragma: no-cache,rate=1.000000,request-context=%u\r\n" \
+    "Pragma: xPlayStrm=1\r\n" \
+    CLIENTGUID \
+    "Pragma: stream-switch-count=%d\r\n" \
+    "Pragma: stream-switch-entry=%s\r\n" \
+    "Connection: Close\r\n\r\n"
 
 /* Unused requests */
 #if 0
-static const char* mmsh_PostRequest =
-    "POST %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s\r\n"
-    "Pragma: client-id=%u\r\n"
+#define mmsh_PostRequest \
+    "POST %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s\r\n" \
+    "Pragma: client-id=%u\r\n" \
 /*    "Pragma: log-line=no-cache,rate=1.000000,stream-time=%u,stream-offset=%u:%u,request-context=2,max-duration=%u\r\n"
- */
-    "Pragma: Content-Length: 0\r\n"
-    CLIENTGUID
-    "\r\n";
+ */ \
+    "Pragma: Content-Length: 0\r\n" \
+    CLIENTGUID \
+    "\r\n"
 
-static const char* mmsh_RangeRequest =
-    "GET %s HTTP/1.0\r\n"
-    "Accept: */*\r\n"
-    USERAGENT
-    "Host: %s:%d\r\n"
-    "Range: bytes=%Lu-\r\n"
-    CLIENTGUID
-    "Connection: Close\r\n\r\n";
+#define mmsh_RangeRequest \
+    "GET %s HTTP/1.0\r\n" \
+    "Accept: */*\r\n" \
+    USERAGENT \
+    "Host: %s:%d\r\n" \
+    "Range: bytes=%Lu-\r\n" \
+    CLIENTGUID \
+    "Connection: Close\r\n\r\n"
 #endif
 
 
@@ -162,8 +160,9 @@ struct mmsh_s {
 
   char          str[SCRATCH_SIZE]; /* scratch buffer to built strings */
 
-  int           stream_type;  /* seekable or broadcast */
-  
+  asf_header_t *asf_header;
+  int           stream_type;  
+
   /* receive buffer */
   
   /* chunk */
@@ -175,53 +174,21 @@ struct mmsh_s {
   int           buf_size;
   int           buf_read;
 
-  uint8_t       asf_header[ASF_HEADER_SIZE];
+  uint8_t       asf_header_buffer[ASF_HEADER_SIZE];
   uint32_t      asf_header_len;
   uint32_t      asf_header_read;
   int           seq_num;
-  int           num_stream_ids;
-  int           stream_ids[ASF_MAX_NUM_STREAMS];
-  int           stream_types[ASF_MAX_NUM_STREAMS];
-  int           packet_length;
-  int64_t       file_length;
-  char          guid[37];
-  uint32_t      bitrates[ASF_MAX_NUM_STREAMS];
-  uint32_t      bitrates_pos[ASF_MAX_NUM_STREAMS];
-
-  int           has_audio;
-  int           has_video;
   
+  int           video_stream;
+  int           audio_stream;
+
   off_t         current_pos;
-  int           user_bandwitdh;
+  int           user_bandwidth;
+  
+  int           playing;
+  unsigned int  start_time;
 };
 
-static int get_guid (unsigned char *buffer, int offset) {
-  int i;
-  GUID g;
-  
-  g.Data1 = LE_32(buffer + offset);
-  g.Data2 = LE_16(buffer + offset + 4);
-  g.Data3 = LE_16(buffer + offset + 6);
-  for(i = 0; i < 8; i++) {
-    g.Data4[i] = buffer[offset + 8 + i];
-  }
-  
-  for (i = 1; i < GUID_END; i++) {
-    if (!memcmp(&g, &guids[i].guid, sizeof(GUID))) {
-      lprintf ("GUID: %s\n", guids[i].name);
-
-      return i;
-    }
-  }
-
-  lprintf ("libmmsh: unknown GUID: 0x%x, 0x%x, 0x%x, "
-           "{ 0x%hx, 0x%hx, 0x%hx, 0x%hx, 0x%hx, 0x%hx, 0x%hx, 0x%hx }\n",
-          g.Data1, g.Data2, g.Data3,
-          g.Data4[0], g.Data4[1], g.Data4[2], g.Data4[3], 
-          g.Data4[4], g.Data4[5], g.Data4[6], g.Data4[7]);
-  return GUID_ERROR;
-}
-
 static int send_command (mmsh_t *this, char *cmd)  {
   int length;
 
@@ -247,7 +214,7 @@ static int get_answer (mmsh_t *this) {
 
   while (!done) {
 
-    if (_x_io_tcp_read(this->stream, this->s, &(this->buf[len]), 1) != 1) {
+    if (_x_io_tcp_read(this->stream, this->s, (char*)&(this->buf[len]), 1) != 1) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                "libmmsh: alert: end of stream\n");
       return 0;
@@ -271,36 +238,36 @@ static int get_answer (mmsh_t *this) {
         int httpver, httpsub, httpcode;
         char httpstatus[51];
 
-        if (sscanf(this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
+        if (sscanf((char*)this->buf, "HTTP/%d.%d %d %50[^\015\012]", &httpver, &httpsub,
             &httpcode, httpstatus) != 4) {
           xine_log (this->stream->xine, XINE_LOG_MSG,
-		    _("libmmsh: bad response format\n"));
+                    _("libmmsh: bad response format\n"));
           return 0;
         }
 
         if (httpcode >= 300 && httpcode < 400) {
           xine_log (this->stream->xine, XINE_LOG_MSG,
-		    _("libmmsh: 3xx redirection not implemented: >%d %s<\n"),
-		    httpcode, httpstatus);
+                    _("libmmsh: 3xx redirection not implemented: >%d %s<\n"),
+                    httpcode, httpstatus);
           return 0;
         }
 
         if (httpcode < 200 || httpcode >= 300) {
           xine_log (this->stream->xine, XINE_LOG_MSG,
-		    _("libmmsh: http status not 2xx: >%d %s<\n"),
-		    httpcode, httpstatus);
+                    _("libmmsh: http status not 2xx: >%d %s<\n"),
+                    httpcode, httpstatus);
           return 0;
         }
       } else {
 
-        if (!strncasecmp(this->buf, "Location: ", 10)) {
+        if (!strncasecmp((char*)this->buf, "Location: ", 10)) {
           xine_log (this->stream->xine, XINE_LOG_MSG,
-		    _("libmmsh: Location redirection not implemented\n"));
+                    _("libmmsh: Location redirection not implemented\n"));
           return 0;
         }
         
-        if (!strncasecmp(this->buf, "Pragma:", 7)) {
-          features = strstr(this->buf + 7, "features=");
+        if (!strncasecmp((char*)this->buf, "Pragma:", 7)) {
+          features = strstr((char*)(this->buf + 7), "features=");
           if (features) {
             if (strstr(features, "seekable")) {
               lprintf("seekable stream\n");
@@ -341,14 +308,14 @@ static int get_chunk_header (mmsh_t *this) {
   lprintf ("get_chunk_header\n");
 
   /* read chunk header */
-  read_len = _x_io_tcp_read(this->stream, this->s, chunk_header, CHUNK_HEADER_LENGTH);
+  read_len = _x_io_tcp_read(this->stream, this->s, (char*)chunk_header, CHUNK_HEADER_LENGTH);
   if (read_len != CHUNK_HEADER_LENGTH) {
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
              "libmmsh: chunk header read failed, %d != %d\n", read_len, CHUNK_HEADER_LENGTH);
     return 0;
   }
-  this->chunk_type       = LE_16 (&chunk_header[0]);
-  this->chunk_length     = LE_16 (&chunk_header[2]);
+  this->chunk_type       = _X_LE_16 (&chunk_header[0]);
+  this->chunk_length     = _X_LE_16 (&chunk_header[2]);
   
   switch (this->chunk_type) {
     case CHUNK_TYPE_DATA:
@@ -368,7 +335,7 @@ static int get_chunk_header (mmsh_t *this) {
   }
   /* read extended header */
   if (ext_header_len > 0) {
-    read_len = _x_io_tcp_read(this->stream, this->s, ext_header, ext_header_len);
+    read_len = _x_io_tcp_read(this->stream, this->s, (char*)ext_header, ext_header_len);
     if (read_len != ext_header_len) {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                "extended header read failed, %d != %d\n", read_len, ext_header_len);
@@ -378,16 +345,16 @@ static int get_chunk_header (mmsh_t *this) {
   
   switch (this->chunk_type) {
     case CHUNK_TYPE_DATA:
-      this->chunk_seq_number = LE_32 (&ext_header[0]);
+      this->chunk_seq_number = _X_LE_32 (&ext_header[0]);
       lprintf ("chunk type:       CHUNK_TYPE_DATA\n");
       lprintf ("chunk length:     %d\n", this->chunk_length);
       lprintf ("chunk seq:        %d\n", this->chunk_seq_number);
       lprintf ("unknown:          %d\n", ext_header[4]);
       lprintf ("mmsh seq:         %d\n", ext_header[5]);
-      lprintf ("len2:             %d\n", LE_16(&ext_header[6]));
+      lprintf ("len2:             %d\n", _X_LE_16(&ext_header[6]));
       break;
     case CHUNK_TYPE_END:
-      this->chunk_seq_number = LE_32 (&ext_header[0]);
+      this->chunk_seq_number = _X_LE_32 (&ext_header[0]);
       lprintf ("chunk type:       CHUNK_TYPE_END\n");
       lprintf ("continue: %d\n", this->chunk_seq_number);
       break;
@@ -397,7 +364,7 @@ static int get_chunk_header (mmsh_t *this) {
       lprintf ("unknown:          %2X %2X %2X %2X %2X %2X\n",
                ext_header[0], ext_header[1], ext_header[2], ext_header[3],
                ext_header[4], ext_header[5]);
-      lprintf ("len2:             %d\n", LE_16(&ext_header[6]));
+      lprintf ("len2:             %d\n", _X_LE_16(&ext_header[6]));
       break;
     case CHUNK_TYPE_RESET:
       lprintf ("chunk type:       CHUNK_TYPE_RESET\n");
@@ -419,7 +386,6 @@ static int get_header (mmsh_t *this) {
   lprintf("get_header\n");
 
   this->asf_header_len = 0;
-  this->asf_header_read = 0;
   
   /* read chunk */
   while (1) {
@@ -430,7 +396,7 @@ static int get_header (mmsh_t *this) {
                    "libmmsh: the asf header exceed %d bytes\n", ASF_HEADER_SIZE);
           return 0;
         } else {
-          len = _x_io_tcp_read(this->stream, this->s, this->asf_header + this->asf_header_len,
+          len = _x_io_tcp_read(this->stream, this->s, (char*)(this->asf_header_buffer + this->asf_header_len),
                              this->chunk_length);
           this->asf_header_len += len;
           if (len != this->chunk_length) {
@@ -448,7 +414,7 @@ static int get_header (mmsh_t *this) {
 
   if (this->chunk_type == CHUNK_TYPE_DATA) {
     /* read the first data chunk */
-    len = _x_io_tcp_read(this->stream, this->s, this->buf, this->chunk_length);
+    len = _x_io_tcp_read(this->stream, this->s, (char*)this->buf, this->chunk_length);
     if (len != this->chunk_length) {
       return 0;
     } else {
@@ -460,122 +426,28 @@ static int get_header (mmsh_t *this) {
   }
 }
 
-static void interp_header (mmsh_t *this) {
-
-  int i;
+static int interp_header (mmsh_t *this) {
 
   lprintf ("interp_header, header_len=%d\n", this->asf_header_len);
 
-  this->packet_length = 0;
-
-  /*
-   * parse asf header
-   */
-
-  i = 30;
-  while ((i + 24) < this->asf_header_len) {
-
-    int guid;
-    uint64_t length;
-
-    guid = get_guid(this->asf_header, i);
-    i += 16;
-
-    length = LE_64(this->asf_header + i);
-    i += 8;
-
-    if ((i + length) >= this->asf_header_len) return;
-
-    switch (guid) {
-
-      case GUID_ASF_FILE_PROPERTIES:
-
-        this->packet_length = LE_32(this->asf_header + i + 92 - 24);
-        this->file_length   = LE_64(this->asf_header + i + 40 - 24);
-        lprintf ("file object, file_length = %lld, packet length = %d",
-		 this->file_length, this->packet_count);
-        break;
-
-      case GUID_ASF_STREAM_PROPERTIES:
-        {
-          uint16_t flags;
-          uint16_t stream_id;
-          int      type;
-          int      encrypted;
-
-          guid = get_guid(this->asf_header, i);
-          switch (guid) {
-            case GUID_ASF_AUDIO_MEDIA:
-              type = ASF_STREAM_TYPE_AUDIO;
-              this->has_audio = 1;
-              break;
-    
-            case GUID_ASF_VIDEO_MEDIA:
-            case GUID_ASF_JFIF_MEDIA:
-            case GUID_ASF_DEGRADABLE_JPEG_MEDIA:
-              type = ASF_STREAM_TYPE_VIDEO;
-              this->has_video = 1;
-              break;
-          
-            case GUID_ASF_COMMAND_MEDIA:
-              type = ASF_STREAM_TYPE_CONTROL;
-              break;
-        
-            default:
-              type = ASF_STREAM_TYPE_UNKNOWN;
-          }
-
-          flags = LE_16(this->asf_header + i + 48);
-          stream_id = flags & 0x7F;
-          encrypted = flags >> 15;
-
-          lprintf ("stream object, stream id: %d, type: %d, encrypted: %d\n",
-                   stream_id, type, encrypted);
-
-          this->stream_types[stream_id] = type;
-          this->stream_ids[this->num_stream_ids] = stream_id;
-          this->num_stream_ids++;
-
-        }
-        break;
-
-      case GUID_ASF_STREAM_BITRATE_PROPERTIES:
-        {
-          uint16_t streams = LE_16(this->asf_header + i);
-          uint16_t stream_id;
-          int j;
-
-	  lprintf ("stream bitrate properties\n");
-          lprintf ("streams %d\n", streams);
-
-          for(j = 0; j < streams; j++) {
-            stream_id = LE_16(this->asf_header + i + 2 + j * 6);
-
-            lprintf ("stream id %d\n", stream_id);
-
-            this->bitrates[stream_id] = LE_32(this->asf_header + i + 4 + j * 6);
-            this->bitrates_pos[stream_id] = i + 4 + j * 6;
-            xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-                     "libmmsh: stream id %d, bitrate %d\n",
-                     stream_id, this->bitrates[stream_id]);
-          }
-        }
-        break;
-
-      default:
-        lprintf ("unknown object\n");
-        break;
-    }
-
-    lprintf ("length    : %lld\n", length);
-
-    if (length > 24) {
-      i += length - 24;
-    }
+  /* delete previous header */
+  if (this->asf_header) {
+    asf_header_delete(this->asf_header);
   }
+
+  /* the header starts with :
+   *   byte  0-15: header guid
+   *   byte 16-23: header length
+   */
+  this->asf_header = asf_header_new(this->asf_header_buffer + 24, this->asf_header_len - 24);
+  if (!this->asf_header)
+    return 0;
+
+  this->buf_size = this->asf_header->file->packet_size;
+  return 1;
 }
 
-const static char *const mmsh_proto_s[] = { "mms", "mmsh", NULL };
+static const char *const mmsh_proto_s[] = { "mms", "mmsh", NULL };
 
 static int mmsh_valid_proto (char *proto) {
   int i = 0;
@@ -645,22 +517,13 @@ static int mmsh_tcp_connect(mmsh_t *this) {
   return 0;
 }
 
-
+/*
+ * firts http request
+ */
 static int mmsh_connect_int(mmsh_t *this, int bandwidth) {
-  int    i;
-  int    video_stream = -1;
-  int    audio_stream = -1;
-  int    max_arate    = -1;
-  int    min_vrate    = -1;
-  int    min_bw_left  = 0;
-  int    stream_id;
-  int    bandwitdh_left;
-  char   stream_selection[10 * ASF_MAX_NUM_STREAMS]; /* 10 chars per stream */
-  int    offset;
   /*
    * let the negotiations begin...
    */
-  this->num_stream_ids = 0;
    
   /* first request */
   lprintf("first http request\n");
@@ -669,151 +532,100 @@ static int mmsh_connect_int(mmsh_t *this, int bandwidth) {
             this->host, this->port, 1);
 
   if (!send_command (this, this->str))
-    goto fail;
+    return 0;
 
   if (!get_answer (this)) 
-    goto fail;
+    return 0;
 
-    
-  get_header(this);
-  interp_header(this);
+  get_header (this); /* FIXME: it returns 0 */
+
+  if (!interp_header (this))
+    return 0;
   
-  close(this->s);
+  close (this->s);
   report_progress (this->stream, 20);
 
+  asf_header_choose_streams (this->asf_header, bandwidth,
+                             &this->video_stream, &this->audio_stream);
+
+  lprintf("audio stream %d, video stream %d\n",
+          this->audio_stream, this->video_stream);
+           
+  asf_header_disable_streams (this->asf_header,
+                              this->video_stream, this->audio_stream);
+                              
+  if (mmsh_tcp_connect(this))
+    return 0;
   
-  /* choose the best quality for the audio stream */
-  /* i've never seen more than one audio stream */
-  for (i = 0; i < this->num_stream_ids; i++) {
-    stream_id = this->stream_ids[i];
-    switch (this->stream_types[stream_id]) {
-      case ASF_STREAM_TYPE_AUDIO:
-        if ((audio_stream == -1) || (this->bitrates[stream_id] > max_arate)) {
-          audio_stream = stream_id;
-          max_arate = this->bitrates[stream_id];
-        }
-        break;
-      default:
-        break;
-    }
-  }
+  return 1;
+}
 
-  /* choose a video stream adapted to the user bandwidth */
-  bandwitdh_left = bandwidth - max_arate;
-  if (bandwitdh_left < 0) {
-    bandwitdh_left = 0;
-  }
-  lprintf("bandwitdh %d, left %d\n", bandwidth, bandwitdh_left);
-
-  min_bw_left = bandwitdh_left;
-  for (i = 0; i < this->num_stream_ids; i++) {
-    stream_id = this->stream_ids[i];
-    switch (this->stream_types[stream_id]) {
-      case ASF_STREAM_TYPE_VIDEO:
-        if (((bandwitdh_left - this->bitrates[stream_id]) < min_bw_left) &&
-            (bandwitdh_left >= this->bitrates[stream_id])) {
-          video_stream = stream_id;
-          min_bw_left = bandwitdh_left - this->bitrates[stream_id];
-        }
-        break;
-      default:
-        break;
-    }
-  }  
-
-  /* choose the stream with the lower bitrate */
-  if ((video_stream == -1) && this->has_video) {
-    for (i = 0; i < this->num_stream_ids; i++) {
-      stream_id = this->stream_ids[i];
-      switch (this->stream_types[stream_id]) {
-        case ASF_STREAM_TYPE_VIDEO:
-          if ((video_stream == -1) || 
-              (this->bitrates[stream_id] < min_vrate) ||
-              (!min_vrate)) {
-            video_stream = stream_id;
-            min_vrate = this->bitrates[stream_id];
-          }
-          break;
-        default:
-          break;
-      }
-    }
-  }
-
-  lprintf("audio stream %d, video stream %d\n", audio_stream, video_stream);
+/*
+ * second http request
+ */
+static int mmsh_connect_int2(mmsh_t *this, int bandwidth) {
+  int    i;
+  char   stream_selection[10 * ASF_MAX_NUM_STREAMS]; /* 10 chars per stream */
+  int    offset;
   
   /* second request */
   lprintf("second http request\n");
 
-  if (mmsh_tcp_connect(this)) {
-    goto fail;
-  }
-
   /* stream selection string */
   /* The same selection is done with mmst */
   /* 0 means selected */
   /* 2 means disabled */
   offset = 0;
-  for (i = 0; i < this->num_stream_ids; i++) {
+  for (i = 0; i < this->asf_header->stream_count; i++) {
     int size;
-    if ((this->stream_ids[i] == audio_stream) ||
-        (this->stream_ids[i] == video_stream)) {
+    if ((i == this->audio_stream) ||
+        (i == this->video_stream)) {
       size = snprintf(stream_selection + offset, sizeof(stream_selection) - offset,
-                      "ffff:%d:0 ", this->stream_ids[i]);
+                      "ffff:%d:0 ", this->asf_header->streams[i]->stream_number);
     } else {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
-               "disabling stream %d\n", this->stream_ids[i]);
+               "disabling stream %d\n", this->asf_header->streams[i]->stream_number);
       size = snprintf(stream_selection + offset, sizeof(stream_selection) - offset,
-                      "ffff:%d:2 ", this->stream_ids[i]);
+                      "ffff:%d:2 ", this->asf_header->streams[i]->stream_number);
     }
-    if (size < 0) goto fail;
+    if (size < 0)
+      return 0;
     offset += size;
   }
 
   switch (this->stream_type) {
     case MMSH_SEEKABLE:
       snprintf (this->str, SCRATCH_SIZE, mmsh_SeekableRequest, this->uri,
-                this->host, this->port, 0, 0, 0, 2, 0,
-                this->num_stream_ids, stream_selection);
+                this->host, this->port, this->start_time, 0, 0, 2, 0,
+                this->asf_header->stream_count, stream_selection);
       break;
     case MMSH_LIVE:
       snprintf (this->str, SCRATCH_SIZE, mmsh_LiveRequest, this->uri,
                 this->host, this->port, 2,
-                this->num_stream_ids, stream_selection);
+                this->asf_header->stream_count, stream_selection);
       break;
   }
 
   if (!send_command (this, this->str))
-    goto fail;
+    return 0;
   
   lprintf("before read \n");
 
   if (!get_answer (this))
-    goto fail;
+    return 0;
 
-  if (!get_header(this))
-    goto fail;
-  interp_header(this);
-  this->buf_size = this->packet_length;
-  
-  for (i = 0; i < this->num_stream_ids; i++) {
-    if ((this->stream_ids[i] != audio_stream) &&
-        (this->stream_ids[i] != video_stream)) {
-      lprintf("disabling stream %d\n", this->stream_ids[i]);
+  if (!get_header (this))
+    return 0;
+    
+#if 0
+  if (!interp_header (this))
+    return 0;
+      
+  asf_header_disable_streams (this->asf_header,
+                              this->video_stream, this->audio_stream);
+#endif
 
-      /* forces the asf demuxer to not choose this stream */
-      if (this->bitrates_pos[this->stream_ids[i]]) {
-        this->asf_header[this->bitrates_pos[this->stream_ids[i]]]     = 0;
-        this->asf_header[this->bitrates_pos[this->stream_ids[i]] + 1] = 0;
-        this->asf_header[this->bitrates_pos[this->stream_ids[i]] + 2] = 0;
-        this->asf_header[this->bitrates_pos[this->stream_ids[i]] + 3] = 0;
-      }
-    }
-  }
   return 1;
-  
-fail:
-  return 0;
 }
 
 mmsh_t *mmsh_connect (xine_stream_t *stream, const char *url, int bandwidth) {
@@ -831,14 +643,10 @@ mmsh_t *mmsh_connect (xine_stream_t *stream, const char *url, int bandwidth) {
   this->s               = -1;
   this->asf_header_len  = 0;
   this->asf_header_read = 0;
-  this->num_stream_ids  = 0;
-  this->packet_length   = 0;
   this->buf_size        = 0;
   this->buf_read        = 0;
-  this->has_audio       = 0;
-  this->has_video       = 0;
   this->current_pos     = 0;
-  this->user_bandwitdh  = bandwidth;
+  this->user_bandwidth  = bandwidth;
 
   report_progress (stream, 0);
   
@@ -858,7 +666,7 @@ mmsh_t *mmsh_connect (xine_stream_t *stream, const char *url, int bandwidth) {
   
   report_progress (stream, 30);
 
-  if (!mmsh_connect_int(this, this->user_bandwitdh))
+  if (!mmsh_connect_int(this, this->user_bandwidth))
     goto fail;
 
   report_progress (stream, 100);
@@ -900,7 +708,7 @@ fail:
 static int get_media_packet (mmsh_t *this) {
   int len = 0;
 
-  lprintf("get_media_packet: this->packet_length: %d\n", this->packet_length);
+  lprintf("get_media_packet: this->packet_length: %d\n", this->asf_header->file->packet_size);
 
   if (get_chunk_header(this)) {
     switch (this->chunk_type) {
@@ -917,8 +725,10 @@ static int get_media_packet (mmsh_t *this) {
         if (mmsh_tcp_connect(this))
           return 0;
 
-        if (!mmsh_connect_int(this, this->user_bandwitdh))
+        if (!mmsh_connect_int(this, this->user_bandwidth))
           return 0;
+          
+        this->playing = 0;
 
         /* mmsh_connect_int reads the first data packet */
         /* this->buf_size is set by mmsh_connect_int */    
@@ -938,7 +748,6 @@ static int get_media_packet (mmsh_t *this) {
         if (!get_header(this))
           return 0;
         interp_header(this);
-        this->buf_size = this->packet_length;
         return 2;
       
       default:
@@ -947,19 +756,18 @@ static int get_media_packet (mmsh_t *this) {
         return 0;
     }
 
-    len = _x_io_tcp_read (this->stream, this->s, this->buf, this->chunk_length);
+    len = _x_io_tcp_read (this->stream, this->s, (char*)this->buf, this->chunk_length);
       
     if (len == this->chunk_length) {
       /* explicit padding with 0 */
-      if (this->chunk_length > this->packet_length) {
+      if (this->chunk_length > this->asf_header->file->packet_size) {
         xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                  "libmmsh: chunk_length(%d) > packet_length(%d)\n",
-                 this->chunk_length, this->packet_length);
+                 this->chunk_length, this->asf_header->file->packet_size);
         return 0;
       }
       memset(this->buf + this->chunk_length, 0,
-             this->packet_length - this->chunk_length);
-      this->buf_size = this->packet_length;
+             this->asf_header->file->packet_size - this->chunk_length);
       return 1;
     } else {
       xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
@@ -971,14 +779,14 @@ static int get_media_packet (mmsh_t *this) {
   }
 }
 
-int mmsh_peek_header (mmsh_t *this, char *data, int maxsize) {
-  int len;
+size_t mmsh_peek_header (mmsh_t *this, char *data, size_t maxsize) {
+  size_t len;
 
   lprintf("mmsh_peek_header\n");
 
   len = (this->asf_header_len < maxsize) ? this->asf_header_len : maxsize;
 
-  memcpy(data, this->asf_header, len);
+  memcpy(data, this->asf_header_buffer, len);
   return len;
 }
 
@@ -994,21 +802,30 @@ int mmsh_read (mmsh_t *this, char *data, int len) {
     if (this->asf_header_read < this->asf_header_len) {
       int n, bytes_left ;
 
-      bytes_left = this->asf_header_len - this->asf_header_read ;
+      bytes_left = this->asf_header_len - this->asf_header_read;
 
       if ((len-total) < bytes_left)
-	n = len-total;
+        n = len-total;
       else
-	n = bytes_left;
+        n = bytes_left;
 
-      xine_fast_memcpy (&data[total], &this->asf_header[this->asf_header_read], n);
+      xine_fast_memcpy (&data[total], &this->asf_header_buffer[this->asf_header_read], n);
 
       this->asf_header_read += n;
       total += n;
       this->current_pos += n;
+      
+      if (this->asf_header_read == this->asf_header_len)
+      	break;
     } else {
 
       int n, bytes_left ;
+      
+      if (!this->playing) {
+        if (!mmsh_connect_int2 (this, this->user_bandwidth))
+          break;
+        this->playing = 1;
+      }
 
       bytes_left = this->buf_size - this->buf_read;
 
@@ -1018,13 +835,13 @@ int mmsh_read (mmsh_t *this, char *data, int len) {
         this->buf_read = 0;
         packet_type = get_media_packet (this);
 
-	if (packet_type == 0) {
-	  xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
+        if (packet_type == 0) {
+          xprintf (this->stream->xine, XINE_VERBOSITY_LOG,
                    "libmmsh: get_media_packet failed\n");
           return total;
-	} else if (packet_type == 2) {
-	  continue;
-	}
+        } else if (packet_type == 2) {
+          continue;
+        }
 
         bytes_left = this->buf_size;
       }
@@ -1063,15 +880,22 @@ void mmsh_close (mmsh_t *this) {
     free(this->password);
   if (this->uri)
     free(this->uri);
+  if (this->asf_header)
+    asf_header_delete(this->asf_header);
   if (this)
     free (this);
 }
 
 
 uint32_t mmsh_get_length (mmsh_t *this) {
-  return this->file_length;
+  return this->asf_header->file->file_size;
 }
 
 off_t mmsh_get_current_pos (mmsh_t *this) {
   return this->current_pos;
 }
+
+void mmsh_set_start_time (mmsh_t *this, int time_offset) {
+  if (time_offset >= 0)
+    this->start_time = time_offset;
+}
