@@ -15,15 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * foovideo.c: This is a reference video decoder for the xine multimedia
  * player. It really works too! It will output frames of packed YUY2 data
  * where each byte in the map is the same value, which is 3 larger than the
  * value from the last frame. This creates a slowly rotating solid color
  * frame when the frames are played in succession.
- *
- * $Id: foovideo.c,v 1.25 2006/06/02 22:18:58 dsalt Exp $
  */
 
 #include <stdio.h>
@@ -98,7 +96,7 @@ static void foovideo_decode_data (video_decoder_t *this_gen,
   }
   
   if (buf->decoder_flags & BUF_FLAG_STDHEADER) { /* need to initialize */
-    this->stream->video_out->open (this->stream->video_out, this->stream);
+    (this->stream->video_out->open) (this->stream->video_out, this->stream);
 
     if(this->buf)
       free(this->buf);
@@ -296,7 +294,7 @@ static const decoder_info_t dec_info_video = {
  * The plugin catalog entry. This is the only information that this plugin
  * will export to the public.
  */
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* { type, API, "name", version, special_info, init_function } */
   { PLUGIN_VIDEO_DECODER, 18, "foovideo", XINE_VERSION_CODE, &dec_info_video, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
