@@ -15,15 +15,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
- * $Id: video_out_opengl.c,v 1.58 2006/06/20 01:07:58 dgp85 Exp $
- * 
  * video_out_opengl.c, OpenGL based interface for xine
  *
  * Written by Matthias Hopf <mat@mshopf.de>,
  * based on the xshm and xv video output plugins.
- * 
  */
 
 /* #define LOG */
@@ -966,7 +963,7 @@ static int render_setup_fp_yuv (opengl_driver_t *this) {
  * List of render backends
  */
 /* name, display, image,  setup, needsrgb, defaction, fallback */
-opengl_render_t opengl_rb[] = {
+static const opengl_render_t opengl_rb[] = {
     {   "2D_Tex_Fragprog",  render_tex2d, render_image_fp_yuv, 
 	render_setup_fp_yuv, 0, RENDER_NONE, 1 },
     {   "2D_Tex",           render_tex2d, render_image_tex,
@@ -1007,7 +1004,7 @@ static void *render_run (opengl_driver_t *this) {
   opengl_frame_t  *frame;
   struct timeval   curtime;
   struct timespec  timeout;
-  opengl_render_t *render;
+  const opengl_render_t *render;
   
   lprintf ("* render thread created\n");
   while (1) {
@@ -1995,7 +1992,7 @@ static const vo_info_t vo_info_opengl = {
  * exported plugin catalog entry
  */
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_VIDEO_OUT, 21, "opengl", XINE_VERSION_CODE, &vo_info_opengl, opengl_init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
