@@ -15,10 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
- * $Id: video_out_vidix.c,v 1.75 2006/12/19 19:10:51 dsalt Exp $
- * 
  * video_out_vidix.c
  *
  * xine video_out driver to vidix library by Miguel Freitas 30/05/2002
@@ -26,7 +24,6 @@
  * based on video_out_xv.c, video_out_syncfb.c and video_out_pgx64.c
  *
  * some vidix specific code from mplayer (file vosub_vidix.c)
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -703,6 +700,18 @@ static int vidix_get_property (vo_driver_t *this_gen, int property) {
     case VO_PROP_WINDOW_HEIGHT:
       this->props[property].value = this->sc.gui_height;
       break;
+    case VO_PROP_OUTPUT_WIDTH:
+      this->props[property].value = this->sc.output_width;
+      break;
+    case VO_PROP_OUTPUT_HEIGHT:
+      this->props[property].value = this->sc.output_height;
+      break;
+    case VO_PROP_OUTPUT_XOFFSET:
+      this->props[property].value = this->sc.output_xoffset;
+      break;
+    case VO_PROP_OUTPUT_YOFFSET:
+      this->props[property].value = this->sc.output_yoffset;
+      break;
   }
   
   lprintf ("video_out_vidix: property #%d = %d\n", property,
