@@ -15,9 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: buffer.h,v 1.153 2005/10/29 19:52:35 tmmm Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  *
  * contents:
@@ -28,7 +26,6 @@
  * free buffer pool management routines
  *
  * FIFO buffer structures/routines
- *
  */
 
 #ifndef HAVE_BUFFER_H
@@ -74,18 +71,19 @@ extern "C" {
 
 /* control buffer types */
 
-#define BUF_CONTROL_BASE          0x01000000
-#define BUF_CONTROL_START         0x01000000
-#define BUF_CONTROL_END           0x01010000
-#define BUF_CONTROL_QUIT          0x01020000
-#define BUF_CONTROL_DISCONTINUITY 0x01030000 /* former AVSYNC_RESET */
-#define BUF_CONTROL_NOP           0x01040000
-#define BUF_CONTROL_AUDIO_CHANNEL 0x01050000
-#define BUF_CONTROL_SPU_CHANNEL   0x01060000
-#define BUF_CONTROL_NEWPTS        0x01070000
-#define BUF_CONTROL_RESET_DECODER 0x01080000
-#define BUF_CONTROL_HEADERS_DONE  0x01090000
-#define BUF_CONTROL_FLUSH_DECODER 0x010a0000
+#define BUF_CONTROL_BASE            0x01000000
+#define BUF_CONTROL_START           0x01000000
+#define BUF_CONTROL_END             0x01010000
+#define BUF_CONTROL_QUIT            0x01020000
+#define BUF_CONTROL_DISCONTINUITY   0x01030000 /* former AVSYNC_RESET */
+#define BUF_CONTROL_NOP             0x01040000
+#define BUF_CONTROL_AUDIO_CHANNEL   0x01050000
+#define BUF_CONTROL_SPU_CHANNEL     0x01060000
+#define BUF_CONTROL_NEWPTS          0x01070000
+#define BUF_CONTROL_RESET_DECODER   0x01080000
+#define BUF_CONTROL_HEADERS_DONE    0x01090000
+#define BUF_CONTROL_FLUSH_DECODER   0x010a0000
+#define BUF_CONTROL_RESET_TRACK_MAP 0x010b0000
 
 /* video buffer types:  (please keep in sync with buffer_types.c) */
 
@@ -180,6 +178,19 @@ extern "C" {
 #define BUF_VIDEO_RT21		0x02570000
 #define BUF_VIDEO_FPS1		0x02580000
 #define BUF_VIDEO_DUCKTM2	0x02590000
+#define BUF_VIDEO_CSCD		0x025A0000
+#define BUF_VIDEO_ALGMM		0x025B0000
+#define BUF_VIDEO_ZMBV		0x025C0000
+#define BUF_VIDEO_AVS		0x025D0000
+#define BUF_VIDEO_SMACKER	0x025E0000
+#define BUF_VIDEO_NUV		0x025F0000
+#define BUF_VIDEO_KMVC		0x02600000
+#define BUF_VIDEO_FLASHSV	0x02610000
+#define BUF_VIDEO_CAVS		0x02620000
+#define BUF_VIDEO_VP6F		0x02630000
+#define BUF_VIDEO_THEORA_RAW	0x02640000
+#define BUF_VIDEO_VC1		0x02650000
+#define BUF_VIDEO_VMNC		0x02660000
 
 /* audio buffer types:  (please keep in sync with buffer_types.c) */
 
@@ -242,6 +253,11 @@ extern "C" {
 #define BUF_AUDIO_SHORTEN	0x03360000
 #define BUF_AUDIO_WESTWOOD_SND1	0x03370000
 #define BUF_AUDIO_WMALL		0x03380000
+#define BUF_AUDIO_TRUESPEECH	0x03390000
+#define BUF_AUDIO_TTA		0x033A0000
+#define BUF_AUDIO_SMACKER	0x033B0000
+#define BUF_AUDIO_FLVADPCM	0x033C0000
+#define BUF_AUDIO_WAVPACK	0x033D0000
 
 /* spu buffer types:    */
 
@@ -284,7 +300,7 @@ struct extra_info_s {
 };
 
 
-#define BUF_NUM_DEC_INFO 4
+#define BUF_NUM_DEC_INFO 5
 
 typedef struct buf_element_s buf_element_t;
 struct buf_element_s {
@@ -590,24 +606,24 @@ struct fifo_buffer_s
  * allocate num_buffers of buf_size bytes each
  */
 
-fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size);
-fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size);
+fifo_buffer_t *_x_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_PROTECTED;
+fifo_buffer_t *_x_dummy_fifo_buffer_new (int num_buffers, uint32_t buf_size) XINE_PROTECTED;
 
 
 /* return BUF_VIDEO_xxx given the fourcc
  * fourcc_int must be read in machine endianness
  * example: fourcc_int = *(uint32_t *)fourcc_char;
  */
-uint32_t _x_fourcc_to_buf_video( uint32_t fourcc_int );
+uint32_t _x_fourcc_to_buf_video( uint32_t fourcc_int ) XINE_PROTECTED;
 
 /* return codec name given BUF_VIDEO_xxx */
-char * _x_buf_video_name( uint32_t buf_type );
+char * _x_buf_video_name( uint32_t buf_type ) XINE_PROTECTED;
 
-/* return BUF_VIDEO_xxx given the formattag */
-uint32_t _x_formattag_to_buf_audio( uint32_t formattag );
+/* return BUF_AUDIO_xxx given the formattag */
+uint32_t _x_formattag_to_buf_audio( uint32_t formattag ) XINE_PROTECTED;
 
-/* return codec name given BUF_VIDEO_xxx */
-char * _x_buf_audio_name( uint32_t buf_type );
+/* return codec name given BUF_AUDIO_xxx */
+char * _x_buf_audio_name( uint32_t buf_type ) XINE_PROTECTED;
 
 
 #ifndef ATTRIBUTE_PACKED
@@ -650,10 +666,10 @@ typedef struct __attribute__((__packed__)) {
 #endif
 
 /* convert xine_bmiheader struct from little endian */
-void _x_bmiheader_le2me( xine_bmiheader *bih );
+void _x_bmiheader_le2me( xine_bmiheader *bih ) XINE_PROTECTED;
 
 /* convert xine_waveformatex struct from little endian */
-void _x_waveformatex_le2me( xine_waveformatex *wavex );
+void _x_waveformatex_le2me( xine_waveformatex *wavex ) XINE_PROTECTED;
 
 #ifdef __cplusplus
 }
