@@ -15,14 +15,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Buffered Input Plugin (request optimizer).
  *
  * The goal of this input plugin is to reduce 
  * the number of calls to the real input plugin.
- *
- * $Id: input_cache.c,v 1.13 2007/01/19 01:05:25 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +34,7 @@
 */
 
 #include "xine_internal.h"
+#include <assert.h>
 
 #define DEFAULT_BUFFER_SIZE 1024
 
@@ -67,14 +66,18 @@ static off_t cache_plugin_read(input_plugin_t *this_gen, char *buf, off_t len) {
   off_t read_len = 0;
   off_t main_read;
 
-  lprintf("cache_plugin_read: len=%lld\n", len);
+  lprintf("cache_plugin_read: len=%"PRId64"\n", len);
   this->read_call++;
 
   /* optimized for common cases */
   if (len <= (this->buf_len - this->buf_pos)) {
     /* all bytes are in the buffer */
     switch (len) {
-#if !(defined(sparc) || defined(__sparc__) || defined __ia64__)
+#if defined(__i386__) || defined(__x86_64__)
+      /* These are restricted to x86 and amd64. Some other architectures don't
+       * handle unaligned accesses in the same way, quite possibly requiring
+       * extra code over and above simple byte copies.
+       */
       case 8:
         *((uint64_t *)buf) = *(uint64_t *)(&(this->buf[this->buf_pos]));
         break;
@@ -188,6 +191,7 @@ static buf_element_t *cache_plugin_read_block(input_plugin_t *this_gen, fifo_buf
     if (buf) {
       buf->type = BUF_DEMUX_BLOCK;
    
+      assert(todo <= buf->max_size);
       read_len = cache_plugin_read (this_gen, buf->content, todo);
       buf->size = read_len;
     }
@@ -205,7 +209,7 @@ static off_t cache_plugin_seek(input_plugin_t *this_gen, off_t offset, int origi
   off_t rel_offset;
   off_t new_buf_pos;
 
-  lprintf("offset: %lld, origin: %d\n", offset, origin);
+  lprintf("offset: %"PRId64", origin: %d\n", offset, origin);
   this->seek_call++;
 
   if( !this->buf_len ) {
@@ -236,7 +240,7 @@ static off_t cache_plugin_seek(input_plugin_t *this_gen, off_t offset, int origi
     }
   
     new_buf_pos = (off_t)this->buf_pos + rel_offset;
-    lprintf("buf_len: %d, rel_offset=%lld, new_buf_pos=%lld\n",
+    lprintf("buf_len: %d, rel_offset=%"PRId64", new_buf_pos=%"PRId64"\n",
   	  this->buf_len, rel_offset, new_buf_pos);
   
     if ((new_buf_pos < 0) || (new_buf_pos >= this->buf_len)) {
