@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,37 +15,42 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: scratch.h,v 1.3 2002/03/01 09:29:50 guenter Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * scratch buffer for log output
- *
  */
 
 #ifndef HAVE_SCRATCH_H
 #define HAVE_SCRATCH_H
 
 #include <stdarg.h>
+#include <pthread.h>
 
 typedef struct scratch_buffer_s scratch_buffer_t;
 
+#define SCRATCH_LINE_LEN_MAX  1024
+
 struct scratch_buffer_s {
 
-  void (*scratch_printf) (scratch_buffer_t *this, const char *format, va_list ap);
+  void
+#if __GNUC__ >= 3
+               __attribute__((__format__(__printf__, 2, 0)))
+#endif
+               (*scratch_printf) (scratch_buffer_t *this, const char *format, va_list ap);
 
-  char **(*get_content) (scratch_buffer_t *this);
+  char       **(*get_content) (scratch_buffer_t *this);
 
-  void (*dispose) (scratch_buffer_t *this);
+  void         (*dispose) (scratch_buffer_t *this);
 
-  char **lines;
-  char **ordered;
+  char         **lines;
+  char         **ordered;
 
-  int    num_lines;
-  int    cur;
+  int            num_lines;
+  int            cur;
 
+  pthread_mutex_t lock;
 };
 
-scratch_buffer_t *new_scratch_buffer (int num_lines);
+scratch_buffer_t *_x_new_scratch_buffer (int num_lines) XINE_PROTECTED;
 
 #endif
